@@ -1,7 +1,7 @@
 use crate::app_version;
 use crate::args::AppArgs;
 #[cfg(feature = "audio")]
-use crate::audio::music_player::MusicPlayer;
+use crate::audio::music_player::{MusicPlayer, MusicPlayerEvent};
 use crate::network::handler::NetworkHandler;
 use crate::{
     core::*,
@@ -14,11 +14,12 @@ use crate::{
         PopupMessage, {UiScreen, UiState},
     },
 };
-use crossterm::event::{KeyCode, KeyModifiers};
 use libp2p::identity::Keypair;
 use libp2p::{gossipsub, swarm::SwarmEvent};
 use rand::SeedableRng;
 use rand_chacha::ChaCha8Rng;
+use ratatui::crossterm;
+use ratatui::crossterm::event::{KeyCode, KeyModifiers};
 use std::time::{Duration, Instant};
 use tokio::sync::mpsc;
 use tokio_util::sync::CancellationToken;
@@ -36,6 +37,8 @@ pub enum AppEvent {
     FastTick(Tick),
     TerminalEvent(TerminalEvent),
     NetworkEvent(SwarmEvent<gossipsub::Event>),
+    #[cfg(feature = "audio")]
+    AudioEvent(MusicPlayerEvent),
 }
 
 #[derive(Debug)]
@@ -94,7 +97,10 @@ impl App {
 
             self.ui
             .push_popup(PopupMessage::Ok{
-               message: "The game was closed during a space adventure.\nAll the cargo and fuel have been lost.\nNext time go back to the base first!".to_string(), is_skippable:false,tick: Tick::now()});
+               message: "The game was closed during a space adventure.\nAll the cargo and fuel have been lost.\nNext time go back to the base first!".to_string(), 
+               is_skippable:false,
+               timestamp: Tick::now()
+            });
         }
 
         const SIMULATION_UPDATE_INTERVAL: Tick = 250 * MILLISECONDS;
@@ -136,7 +142,7 @@ impl App {
                     self.ui.push_popup(PopupMessage::Ok {
                         message,
                         is_skippable: true,
-                        tick: Tick::now(),
+                        timestamp: Tick::now(),
                     });
                 }
                 Ok(None) => {}
@@ -328,6 +334,17 @@ impl App {
                     AppEvent::NetworkEvent(swarm_event) => {
                         self.handle_network_events(swarm_event)?;
                     }
+
+                    #[cfg(feature = "audio")]
+                    AppEvent::AudioEvent(audio_event) => match audio_event {
+                        MusicPlayerEvent::StreamOk => {}
+                        MusicPlayerEvent::StreamErr { error_message } => {
+                            self.ui.push_popup(PopupMessage::Error {
+                                message: format!("Music player error: {error_message}"),
+                                timestamp: Tick::now(),
+                            });
+                        }
+                    },
                 }
             }
         }
@@ -353,7 +370,7 @@ impl App {
                 self.ui.push_popup(PopupMessage::Ok {
                     message,
                     is_skippable: false,
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
                 self.new_version_notified = true;
             }
@@ -438,14 +455,14 @@ impl App {
                             self.ui.push_popup(PopupMessage::Ok {
                                 message,
                                 is_skippable: true,
-                                tick: Tick::now(),
+                                timestamp: Tick::now(),
                             });
                         }
                         Ok(None) => {}
                         Err(e) => {
                             self.ui.push_popup(PopupMessage::Error {
                                 message: e.to_string(),
-                                tick: Tick::now(),
+                                timestamp: Tick::now(),
                             });
                         }
                     }
@@ -454,7 +471,7 @@ impl App {
             Err(e) => {
                 self.ui.push_popup(PopupMessage::Error {
                     message: format!("Tick error\n{e}"),
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
             }
         }
@@ -473,14 +490,14 @@ impl App {
                             self.ui.push_popup(PopupMessage::Ok {
                                 message,
                                 is_skippable: true,
-                                tick: Tick::now(),
+                                timestamp: Tick::now(),
                             });
                         }
                         Ok(None) => {}
                         Err(e) => {
                             self.ui.push_popup(PopupMessage::Error {
                                 message: e.to_string(),
-                                tick: Tick::now(),
+                                timestamp: Tick::now(),
                             });
                         }
                     }
@@ -489,7 +506,7 @@ impl App {
             Err(e) => {
                 self.ui.push_popup(PopupMessage::Error {
                     message: format!("Tick error\n{e}"),
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
             }
         }
@@ -586,9 +603,7 @@ impl App {
         let mut should_draw = false;
         match key_event.code {
             // Exit application directly on `Ctrl-C`. `Esc` asks for confirmation first.
-            KeyCode::Char('c') | KeyCode::Char('C')
-                if key_event.modifiers == KeyModifiers::CONTROL =>
-            {
+            KeyCode::Char('c' | 'C') if key_event.modifiers == KeyModifiers::CONTROL => {
                 self.quit()?;
             }
             _ => {
@@ -598,14 +613,14 @@ impl App {
                             self.ui.push_popup(PopupMessage::Ok {
                                 message,
                                 is_skippable: true,
-                                tick: Tick::now(),
+                                timestamp: Tick::now(),
                             });
                         }
                         Ok(None) => {}
                         Err(e) => {
                             self.ui.push_popup(PopupMessage::Error {
                                 message: e.to_string(),
-                                tick: Tick::now(),
+                                timestamp: Tick::now(),
                             });
                         }
                     }
@@ -631,14 +646,14 @@ impl App {
                     self.ui.push_popup(PopupMessage::Ok {
                         message,
                         is_skippable: true,
-                        tick: Tick::now(),
+                        timestamp: Tick::now(),
                     });
                 }
                 Ok(None) => {}
                 Err(e) => {
                     self.ui.push_popup(PopupMessage::Error {
                         message: e.to_string(),
-                        tick: Tick::now(),
+                        timestamp: Tick::now(),
                     });
                 }
             }
@@ -657,7 +672,7 @@ impl App {
                     self.ui.push_popup(PopupMessage::Ok {
                         message,
                         is_skippable: true,
-                        tick: Tick::now(),
+                        timestamp: Tick::now(),
                     });
                 }
                 Ok(None) => {}
