@@ -1,8 +1,9 @@
 use crate::{
+    core::MAX_SKILL,
     game_engine::MIN_TIREDNESS_FOR_ROLL_DECLINE,
     types::{PlanetId, TeamId, Tick},
 };
-use once_cell::sync::Lazy;
+use std::sync::LazyLock;
 use uuid::uuid;
 
 // DEBUG_TIME_MULTIPLIER should be between 1 and 1000;
@@ -42,6 +43,8 @@ pub const EXPERIENCE_PER_SKILL_MULTIPLIER: f32 = 0.00001;
 pub const MAX_SKILL_INCREASE_PER_LONG_TICK: f32 = 0.35;
 pub const SKILL_DECREMENT_PER_LONG_TICK: f32 = -0.06;
 
+pub const TOURNAMENT_GAME_TRAINING_BONUS_MODIFIER: f32 = 0.5;
+
 pub const REPUTATION_PER_EXPERIENCE: f32 = 0.0001;
 pub const REPUTATION_DECREASE_PER_LONG_TICK: f32 = -0.15;
 pub const MIN_REPUTATION_PER_HONOUR: f32 = 1.0;
@@ -58,6 +61,8 @@ pub const SPECIAL_TRAIT_VALUE_BONUS: f32 = 1.35;
 pub const AUTO_GENERATE_GAMES_NUMBER: usize = 3;
 pub const MAX_AVG_TIREDNESS_PER_AUTO_GAME: f32 = 2.0;
 pub const MAX_AVG_TIREDNESS_PER_CHALLENGED_GAME: f32 = MIN_TIREDNESS_FOR_ROLL_DECLINE;
+pub const SPACE_ADVENTURE_TIREDNESS_COST: f32 = TirednessCost::CRITICAL;
+pub const MAX_AVG_TIREDNESS_PER_SPACE_ADVENTURE: f32 = MAX_SKILL - SPACE_ADVENTURE_TIREDNESS_COST;
 
 pub const BASE_DISTANCES: [u64; 5] = [
     LIGHT_YEAR,
@@ -99,11 +104,11 @@ pub const NETWORK_GAME_START_DELAY: Tick = 30 * SECONDS;
 static GALAXY_ROOT_STR: &str = "71a43700-0000-0000-0000-000000000000";
 static DEFAULT_PLANET_STR: &str = "71a43700-0000-0000-0002-000000000000";
 static SOL_STR: &str = "71a43700-0000-0000-0001-000000000000";
-pub static GALAXY_ROOT_ID: Lazy<PlanetId> =
-    Lazy::new(|| PlanetId::try_parse(GALAXY_ROOT_STR).unwrap());
-pub static DEFAULT_PLANET_ID: Lazy<PlanetId> =
-    Lazy::new(|| PlanetId::try_parse(DEFAULT_PLANET_STR).unwrap());
-pub static SOL_ID: Lazy<PlanetId> = Lazy::new(|| PlanetId::try_parse(SOL_STR).unwrap());
+pub static GALAXY_ROOT_ID: LazyLock<PlanetId> =
+    LazyLock::new(|| PlanetId::try_parse(GALAXY_ROOT_STR).unwrap());
+pub static DEFAULT_PLANET_ID: LazyLock<PlanetId> =
+    LazyLock::new(|| PlanetId::try_parse(DEFAULT_PLANET_STR).unwrap());
+pub static SOL_ID: LazyLock<PlanetId> = LazyLock::new(|| PlanetId::try_parse(SOL_STR).unwrap());
 
 pub struct TirednessCost;
 impl TirednessCost {
@@ -140,7 +145,7 @@ impl ReputationModifier {
     pub const MEDIUM_BONUS: f32 = 0.2;
     pub const HIGH_BONUS: f32 = 0.5;
     pub const SEVERE_BONUS: f32 = 1.5;
-    pub const BONUS_PER_DISTANCE: f32 = 0.000002;
+    pub const BONUS_PER_DISTANCE: f32 = 0.000_002;
 }
 
 pub const MORALE_DECREASE_PER_LONG_TICK: f32 = MoraleModifier::MEDIUM_MALUS;
