@@ -1,10 +1,11 @@
-use std::collections::HashMap;
-
 use crate::{
     app_version,
-    core::{Planet, Rated, Skill, Team, MINUTES, MIN_SKILL, SECONDS},
+    core::{
+        utils::is_default, Planet, Rated, Skill, Team, TickInterval, MINUTES, MIN_SKILL, SECONDS,
+    },
     game_engine::{
         game::{Game, GameSummary},
+        timer,
         types::TeamInGame,
     },
     types::{
@@ -18,6 +19,7 @@ use rand::{seq::SliceRandom, Rng, SeedableRng};
 use rand_chacha::ChaCha8Rng;
 use serde::{Deserialize, Serialize};
 use serde_repr::{Deserialize_repr, Serialize_repr};
+use std::collections::HashMap;
 use strum::Display;
 
 pub type TournamentId = uuid::Uuid;
@@ -29,7 +31,9 @@ pub struct TournamentSummary {
     kartoffel_id: KartoffelId,
     pub organizer_id: TeamId,
     max_participants: usize,
-    pub participants: HashMap<TeamId, TeamInGame>,
+    #[serde(skip_serializing_if = "is_default")]
+    #[serde(default)]
+    pub participant_ids: Vec<TeamId>,
     pub game_ids: Vec<GameId>,
     pub planet_id: PlanetId,
     planet_name: String,
@@ -37,20 +41,18 @@ pub struct TournamentSummary {
     registrations_closing_at: Tick,
     pub ended_at: Option<Tick>,
     pub winner: Option<TeamId>,
+    #[serde(skip_serializing_if = "is_default")]
+    #[serde(default)]
+    pub winner_name: String,
+    #[serde(skip_serializing_if = "is_default")]
+    #[serde(default)]
+    tournament_rating: Skill,
     app_version: [usize; 3],
 }
 
 impl Rated for TournamentSummary {
     fn rating(&self) -> Skill {
-        if self.participants.is_empty() {
-            return MIN_SKILL;
-        }
-
-        self.participants
-            .values()
-            .map(|team| team.rating())
-            .sum::<Skill>()
-            / self.participants.len() as Skill
+        self.tournament_rating
     }
 }
 
@@ -62,7 +64,7 @@ impl TournamentSummary {
             kartoffel_id: tournament.kartoffel_id,
             organizer_id: tournament.organizer_id,
             max_participants: tournament.max_participants,
-            participants: tournament.participants.clone(),
+            participant_ids: tournament.participants.keys().copied().collect(),
             game_ids: tournament.games.iter().map(|g| g.id).collect(),
             planet_id: tournament.planet_id,
             planet_name: tournament.planet_name.clone(),
@@ -70,6 +72,18 @@ impl TournamentSummary {
             registrations_closing_at: tournament.registrations_closing_at,
             ended_at: tournament.ended_at,
             winner: tournament.winner,
+            winner_name: tournament
+                .winner
+                .map(|id| {
+                    tournament
+                        .participants
+                        .get(&id)
+                        .expect("Winner should be a participant")
+                        .name
+                        .clone()
+                })
+                .expect("Ended tournament should have a winner"),
+            tournament_rating: tournament.rating(),
             app_version: tournament.app_version,
         }
     }
@@ -396,6 +410,7 @@ impl Tournament {
         &mut self,
         current_tick: Tick,
         games: &GameMap,
+        past_games: &GameSummaryMap,
     ) -> AppResult<Vec<Game>> {
         if !self.is_initialized() {
             return Err(anyhow!("Tournament should have been initialized."));
@@ -418,13 +433,39 @@ impl Tournament {
         }
 
         let mut tournament_games = vec![];
-
+        let mut generated_past_games = 0;
         for game in self.games.iter() {
             if let Some(world_game) = games.get(&game.id) {
                 tournament_games.push(world_game);
+            } else if past_games.contains_key(&game.id) {
+                // Game completed and was cleaned up - this is expected
+                generated_past_games += 1;
+            } else {
+                log::warn!(
+                    "Tournament game {} not found in games or past_games",
+                    game.id
+                );
+                generated_past_games += 1;
             }
         }
 
+        // This is some sort of extra stop to ensure we don't generate extra games.
+        // FIXME: Not sure why this is necessary...
+        if generated_past_games == self.participants.len() - 1 {
+            if self.pending_team_for_next_game.is_none() {
+                return Err(anyhow!(
+                    "There should be a pending team if there are no available tournament games."
+                ));
+            }
+            self.winner = self.pending_team_for_next_game;
+            self.ended_at = Some(current_tick);
+            log::info!(
+                "Tournament {} is over: reached max number of games.",
+                self.id
+            );
+            return Ok(vec![]);
+        }
+
         // At this point, tournament_games can be empty only if the last game was the final,
         // in which case the pending_team_for_next_game is the tournament winner
         if tournament_games.is_empty() {
@@ -435,6 +476,7 @@ impl Tournament {
             }
             self.winner = self.pending_team_for_next_game;
             self.ended_at = Some(current_tick);
+            log::info!("Tournament {} is over: no available games.", self.id);
             return Ok(vec![]);
         }
 
@@ -461,14 +503,16 @@ impl Tournament {
                     .get(&winner_team_id)
                     .expect("Team should be a participant");
 
-                let game = self.new_game(
+                let new_game = self.new_game(
                     rng,
                     home_team_in_game.clone(),
                     away_team_in_game.clone(),
-                    current_tick + self.game_time_interval,
+                    game.starting_at
+                        + timer::MAX_TIME as Tick * TickInterval::SHORT
+                        + self.game_time_interval,
                 );
-                self.games.push(game.clone());
-                new_games.push(game);
+                self.games.push(new_game.clone());
+                new_games.push(new_game);
 
                 self.pending_team_for_next_game = None;
             } else {
@@ -518,19 +562,24 @@ impl Rated for Tournament {
 #[cfg(test)]
 mod tests {
 
-    use crate::core::{Player, Team, TeamLocation, TickInterval, MAX_PLAYERS_PER_GAME};
-    use crate::game_engine::{Tournament, TournamentType};
-    use crate::types::{AppResult, GameMap, PlanetId, PlayerMap, SystemTimeTick, TeamId, Tick};
+    use crate::core::{Player, Team, TeamLocation, TickInterval, MAX_PLAYERS_PER_GAME, SECONDS};
+    use crate::game_engine::game::GameSummary;
+    use crate::game_engine::{Tournament, TournamentState, TournamentType};
+    use crate::types::{
+        AppResult, GameMap, GameSummaryMap, PlanetId, PlayerMap, SystemTimeTick, TeamId, Tick,
+    };
     use itertools::Itertools;
     use libp2p::PeerId;
 
     #[test]
     fn test_tournament_determinism() -> AppResult<()> {
         let mut tournament = Tournament::test(6, 8);
+        tournament.registrations_closing_at = Tick::now() + 10 * SECONDS;
         let mut replay_tournament = tournament.clone();
 
         fn process_tournament(tournament: &mut Tournament) -> AppResult<()> {
             let mut games = GameMap::new();
+            let mut past_games = GameSummaryMap::new();
 
             for game in tournament.initialize() {
                 games.insert(game.id, game);
@@ -545,7 +594,12 @@ mod tests {
                     }
                 }
 
-                let new_games = tournament.generate_next_games(current_tick, &games)?;
+                let new_games =
+                    tournament.generate_next_games(current_tick, &games, &past_games)?;
+
+                for game in games.values().filter(|g| g.has_ended()) {
+                    past_games.insert(game.id, GameSummary::from_game(game));
+                }
 
                 games.retain(|_, g| !g.has_ended());
 
@@ -575,6 +629,7 @@ mod tests {
             tournament.get_rng_seed(tournament.starting_at() as u64)
         );
         let mut games = GameMap::new();
+        let mut past_game_summaries = GameSummaryMap::new();
         let mut past_games = GameMap::new();
 
         for game in tournament.initialize() {
@@ -587,9 +642,11 @@ mod tests {
                 game.tick(current_tick);
             }
 
-            let new_games = tournament.generate_next_games(current_tick, &games)?;
+            let new_games =
+                tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
 
             for game in games.values().filter(|g| g.has_ended()) {
+                past_game_summaries.insert(game.id, GameSummary::from_game(game));
                 past_games.insert(game.id, game.clone());
             }
 
@@ -704,4 +761,327 @@ mod tests {
         ));
         Ok(())
     }
+
+    #[test]
+    fn test_cancellation_resets_state() -> AppResult<()> {
+        let mut tournament = Tournament::test(4, 4);
+        assert!(!tournament.is_canceled());
+        assert_eq!(tournament.state(Tick::now()), TournamentState::Registration);
+
+        tournament.cancel();
+
+        assert!(tournament.is_canceled());
+        assert_eq!(tournament.state(Tick::now()), TournamentState::Canceled);
+        Ok(())
+    }
+
+    #[test]
+    fn test_generate_next_games_with_past_games_fallback() -> AppResult<()> {
+        // This test verifies that generate_next_games works correctly when
+        // some completed games have been moved from `games` to `past_games`.
+        // The flow is: generate_next_games processes ended games in `games` first,
+        // then ended games are moved to past_games. On subsequent ticks, those games
+        // are only in past_games but should still be correctly counted.
+        let mut tournament = Tournament::test(6, 8);
+        tournament.registrations_closing_at = 0;
+        let mut games = GameMap::new();
+        let mut past_game_summaries = GameSummaryMap::new();
+
+        for game in tournament.initialize() {
+            games.insert(game.id, game);
+        }
+
+        let mut current_tick: Tick = 0;
+        let mut moved_to_past = false;
+
+        while !tournament.has_ended() {
+            for game in games.values_mut() {
+                game.tick(current_tick);
+            }
+
+            // Call generate_next_games with current games + past_games
+            let new_games =
+                tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
+
+            // Move ended games to past_game_summaries (simulating world cleanup)
+            let ended_ids: Vec<_> = games
+                .values()
+                .filter(|g| g.has_ended())
+                .map(|g| g.id)
+                .collect();
+
+            for id in &ended_ids {
+                if let Some(game) = games.get(id) {
+                    past_game_summaries.insert(*id, GameSummary::from_game(game));
+                    if !moved_to_past {
+                        moved_to_past = true;
+                    }
+                }
+            }
+
+            games.retain(|_, g| !g.has_ended());
+
+            for game in new_games {
+                games.insert(game.id, game);
+            }
+
+            current_tick += TickInterval::SHORT;
+        }
+
+        assert!(moved_to_past, "Should have moved at least one game to past_games");
+        assert!(tournament.winner.is_some());
+        // All tournament games should be accounted for in past_game_summaries
+        assert_eq!(
+            past_game_summaries.len(),
+            tournament.participants.len() - 1,
+            "All tournament games should be in past_game_summaries"
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_game_scheduling_uses_starting_at_plus_max_time() -> AppResult<()> {
+        use crate::game_engine::timer;
+
+        let mut tournament = Tournament::test(4, 4);
+        tournament.registrations_closing_at = 0;
+        let mut games = GameMap::new();
+        let mut past_game_summaries = GameSummaryMap::new();
+
+        for game in tournament.initialize() {
+            games.insert(game.id, game);
+        }
+
+        let mut current_tick: Tick = 0;
+        let mut found_new_game = false;
+
+        while !tournament.has_ended() && !found_new_game {
+            for game in games.values_mut() {
+                game.tick(current_tick);
+            }
+
+            // Collect starting_at for games that just ended (before generate_next_games consumes them)
+            let ended_starting_ats: Vec<Tick> = games
+                .values()
+                .filter(|g| g.has_ended())
+                .map(|g| g.starting_at)
+                .collect();
+
+            let new_games =
+                tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
+
+            for game in games.values().filter(|g| g.has_ended()) {
+                past_game_summaries.insert(game.id, GameSummary::from_game(game));
+            }
+
+            games.retain(|_, g| !g.has_ended());
+
+            for game in &new_games {
+                // The new game should be scheduled at:
+                // ended_game.starting_at + MAX_TIME (as Tick) + game_time_interval
+                let game_time_interval = 30 * crate::core::MINUTES;
+                let matches_any = ended_starting_ats.iter().any(|&ended_start| {
+                    let expected = ended_start
+                        + timer::MAX_TIME as Tick * TickInterval::SHORT
+                        + game_time_interval;
+                    game.starting_at == expected
+                });
+                assert!(
+                    matches_any,
+                    "New game starting_at {} should be based on an ended game's starting_at + MAX_TIME + interval",
+                    game.starting_at
+                );
+                found_new_game = true;
+            }
+
+            for game in new_games {
+                games.insert(game.id, game);
+            }
+
+            current_tick += TickInterval::SHORT;
+        }
+
+        assert!(found_new_game, "Should have found at least one new game");
+        Ok(())
+    }
+
+    #[test]
+    fn test_tournament_with_2_participants() -> AppResult<()> {
+        let mut tournament = Tournament::test(2, 4);
+        tournament.registrations_closing_at = 0;
+        let mut games = GameMap::new();
+        let mut past_game_summaries = GameSummaryMap::new();
+
+        let initial_games = tournament.initialize();
+        assert_eq!(initial_games.len(), 1, "2 participants should produce 1 initial game");
+
+        for game in initial_games {
+            games.insert(game.id, game);
+        }
+
+        let mut current_tick: Tick = 0;
+        while !tournament.has_ended() {
+            for game in games.values_mut() {
+                game.tick(current_tick);
+            }
+
+            let new_games =
+                tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
+
+            for game in games.values().filter(|g| g.has_ended()) {
+                past_game_summaries.insert(game.id, GameSummary::from_game(game));
+            }
+
+            games.retain(|_, g| !g.has_ended());
+
+            for game in new_games {
+                games.insert(game.id, game);
+            }
+
+            current_tick += TickInterval::SHORT;
+        }
+
+        assert!(tournament.winner.is_some());
+        assert_eq!(tournament.games.len(), 1, "2 participants = 1 total game");
+        Ok(())
+    }
+
+    #[test]
+    fn test_tournament_odd_participants_bye_handling() -> AppResult<()> {
+        for num_participants in [3, 5, 7] {
+            let mut tournament = Tournament::test(num_participants, 8);
+            tournament.registrations_closing_at = 0;
+            let mut games = GameMap::new();
+            let mut past_game_summaries = GameSummaryMap::new();
+
+            for game in tournament.initialize() {
+                games.insert(game.id, game);
+            }
+
+            let mut current_tick: Tick = 0;
+            while !tournament.has_ended() {
+                for game in games.values_mut() {
+                    game.tick(current_tick);
+                }
+
+                let new_games =
+                    tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
+
+                for game in games.values().filter(|g| g.has_ended()) {
+                    past_game_summaries.insert(game.id, GameSummary::from_game(game));
+                }
+
+                games.retain(|_, g| !g.has_ended());
+
+                for game in new_games {
+                    games.insert(game.id, game);
+                }
+
+                current_tick += TickInterval::SHORT;
+            }
+
+            assert!(
+                tournament.winner.is_some(),
+                "Tournament with {num_participants} participants should have a winner"
+            );
+            assert_eq!(
+                tournament.games.len(),
+                num_participants - 1,
+                "Tournament with {num_participants} participants should have {} total games",
+                num_participants - 1
+            );
+        }
+        Ok(())
+    }
+
+    #[test]
+    fn test_tournament_max_participants_cup() -> AppResult<()> {
+        let max = TournamentType::Cup.max_participants(); // 4
+        let mut tournament = Tournament::test(max, max);
+        tournament.registrations_closing_at = 0;
+        let mut games = GameMap::new();
+        let mut past_game_summaries = GameSummaryMap::new();
+
+        for game in tournament.initialize() {
+            games.insert(game.id, game);
+        }
+
+        let mut current_tick: Tick = 0;
+        while !tournament.has_ended() {
+            for game in games.values_mut() {
+                game.tick(current_tick);
+            }
+
+            let new_games =
+                tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
+
+            for game in games.values().filter(|g| g.has_ended()) {
+                past_game_summaries.insert(game.id, GameSummary::from_game(game));
+            }
+
+            games.retain(|_, g| !g.has_ended());
+
+            for game in new_games {
+                games.insert(game.id, game);
+            }
+
+            current_tick += TickInterval::SHORT;
+        }
+
+        assert!(tournament.winner.is_some());
+        assert_eq!(tournament.games.len(), max - 1);
+        Ok(())
+    }
+
+    #[test]
+    fn test_tournament_max_participants_supercup() -> AppResult<()> {
+        let max = TournamentType::Supercup.max_participants(); // 8
+        let mut tournament = Tournament::test(max, max);
+        tournament.registrations_closing_at = 0;
+        let mut games = GameMap::new();
+        let mut past_game_summaries = GameSummaryMap::new();
+
+        for game in tournament.initialize() {
+            games.insert(game.id, game);
+        }
+
+        let mut current_tick: Tick = 0;
+        while !tournament.has_ended() {
+            for game in games.values_mut() {
+                game.tick(current_tick);
+            }
+
+            let new_games =
+                tournament.generate_next_games(current_tick, &games, &past_game_summaries)?;
+
+            for game in games.values().filter(|g| g.has_ended()) {
+                past_game_summaries.insert(game.id, GameSummary::from_game(game));
+            }
+
+            games.retain(|_, g| !g.has_ended());
+
+            for game in new_games {
+                games.insert(game.id, game);
+            }
+
+            current_tick += TickInterval::SHORT;
+        }
+
+        assert!(tournament.winner.is_some());
+        assert_eq!(tournament.games.len(), max - 1);
+        Ok(())
+    }
+
+    #[test]
+    fn test_canceled_tournament_state() {
+        let mut tournament = Tournament::test(4, 4);
+        assert!(!tournament.is_canceled());
+
+        tournament.cancel();
+
+        assert!(tournament.is_canceled());
+        assert_eq!(tournament.state(Tick::now()), TournamentState::Canceled);
+        // Canceled tournament should have no games generated
+        assert!(tournament.games.is_empty());
+    }
 }
