@@ -1,4 +1,5 @@
 use super::challenge::Challenge;
+use super::handler::NetworkHandler;
 use super::trade::Trade;
 use super::types::{NetworkData, NetworkGame, NetworkRequestState, NetworkTeam, SeedInfo};
 use crate::app_version;
@@ -21,11 +22,6 @@ use rand_chacha::ChaCha8Rng;
 
 #[derive(Debug, Clone)]
 pub enum NetworkCallback {
-    PushSwarmPanelChat {
-        timestamp: Tick,
-        peer_id: PeerId,
-        text: String,
-    },
     PushSwarmPanelLog {
         timestamp: Tick,
         peer_id: Option<PeerId>,
@@ -55,14 +51,6 @@ pub enum NetworkCallback {
     },
 }
 impl NetworkCallback {
-    fn push_swarm_panel_message(timestamp: Tick, peer_id: PeerId, text: String) -> AppCallback {
-        Box::new(move |app: &mut App| {
-            app.ui
-                .push_chat_event(timestamp, Some(peer_id), text.clone());
-            Ok(None)
-        })
-    }
-
     fn bind_address(address: Multiaddr) -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui.push_log_event(
@@ -149,12 +137,14 @@ impl NetworkCallback {
     }
 
     fn handle_message_topic(
-        peer_id: Option<PeerId>,
+        peer_id: PeerId,
         timestamp: Tick,
+        author: String,
         message: String,
     ) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.ui.push_chat_event(timestamp, peer_id, message.clone());
+            app.ui
+                .push_chat_event(timestamp, peer_id, author.clone(), message.clone());
             Ok(None)
         })
     }
@@ -169,7 +159,7 @@ impl NetworkCallback {
                 app.ui.push_popup(PopupMessage::Ok {
                     message: message.clone(),
                     is_skippable: false,
-                    tick: timestamp,
+                    timestamp,
                 });
             }
 
@@ -235,151 +225,165 @@ impl NetworkCallback {
                 log::Level::Error,
             );
         }
-        Box::new(move |app: &mut App| {
-            let tournament = if let Some(t) = app.world.tournaments.get_mut(&tournament_id) {
-                t
-            } else {
-                return Err(anyhow!("No tournament with id {tournament_id} found."));
-            };
 
-            // Check if own_team is tournament organizer.
+        fn send_and_log(
+            network_handler: &mut NetworkHandler,
+            ui: &mut UiScreen,
+            tournament_id: TournamentId,
+            team_id: TeamId,
+            state: TournamentRequestState,
+            msg: String,
+            level: log::Level,
+        ) -> AppResult<()> {
+            network_handler.send_tournament_registration_request(
+                tournament_id,
+                team_id,
+                None,
+                state,
+            )?;
+            ui.push_log_event(Tick::now(), None, msg, level);
+            Ok(())
+        }
+
+        Box::new(move |app: &mut App| {
+            let tournament = app
+                .world
+                .tournaments
+                .get_mut(&tournament_id)
+                .ok_or_else(|| anyhow!("No tournament with id {tournament_id} found."))?;
+
+            // ---- Organizer: handle incoming requests from other teams ----
             if tournament.organizer_id == app.world.own_team_id {
                 assert!(team_id != app.world.own_team_id);
-                // Only possible combinations are
-                // TournamentState::Registration && state = TournamentRequestState::RegistrationRequest
-                // TournamentState::Confirmation  && state = TournamentRequestState::ParticipationRequest|Declined
                 let timestamp = Tick::now();
                 match tournament.state(timestamp) {
                     TournamentState::Registration => {
-                        // We process only requests with state = TournamentRequestState::RegistrationRequest
                         if request_state != TournamentRequestState::RegistrationRequest {
                             let reason =
                                 format!("Invalid tournament request received: {request_state}");
-                            app.network_handler.send_tournament_registration_request(
+                            send_and_log(
+                                &mut app.network_handler,
+                                &mut app.ui,
                                 tournament_id,
                                 team_id,
-                                None,
                                 TournamentRequestState::RegistrationDeclined {
                                     reason: reason.clone(),
                                 },
+                                format!("Registration to tournament {tournament_id} declined for team {team_id}: {reason}."),
+                                log::Level::Error,
                             )?;
-                            app.ui.push_log_event(
-                        Tick::now(),
-                            None,
-                            format!(
-                                "Registration to tournament {tournament_id} declined for team {team_id}: {reason}."
-                            ),
-                            log::Level::Error,
-                        );
+                            return Ok(None);
                         }
 
                         let (team, players) = if let Some(data) = team_data.as_ref() {
                             data
                         } else {
-                            let reason = "Missing team data".to_string();
-                            app.network_handler.send_tournament_registration_request(
+                            send_and_log(
+                                &mut app.network_handler,
+                                &mut app.ui,
                                 tournament_id,
                                 team_id,
-                                None,
                                 TournamentRequestState::RegistrationDeclined {
-                                    reason: reason.clone(),
+                                    reason: "Missing team data".to_string(),
                                 },
+                                format!("Registration to tournament {tournament_id} declined for team {team_id}: Missing team data."),
+                                log::Level::Warn,
                             )?;
-                            app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} declined for team {team_id}: {reason}."), log::Level::Warn);
                             return Ok(None);
                         };
 
                         match tournament.register_team(team, players.clone(), Tick::now()) {
                             Ok(()) => {
-                                app.network_handler.send_tournament_registration_request(
+                                send_and_log(
+                                    &mut app.network_handler,
+                                    &mut app.ui,
                                     tournament_id,
                                     team_id,
-                                    None,
                                     TournamentRequestState::RegistrationOk,
+                                    format!("Registration to tournament {tournament_id} succesful for team {team_id}."),
+                                    log::Level::Info,
                                 )?;
-                                app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} succesful for team {team_id}."), log::Level::Info);
                                 app.network_handler.send_tournament(tournament.clone())?;
                             }
-
                             Err(err) => {
-                                app.network_handler.send_tournament_registration_request(
+                                send_and_log(
+                                    &mut app.network_handler,
+                                    &mut app.ui,
                                     tournament_id,
                                     team_id,
-                                    None,
                                     TournamentRequestState::RegistrationDeclined {
                                         reason: err.to_string(),
                                     },
+                                    format!("Registration to tournament {tournament_id} declined for team {team_id}: {err}."),
+                                    log::Level::Warn,
                                 )?;
-                                app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} declined for team {team_id}: {err}."), log::Level::Warn);
                             }
                         }
                     }
 
-                    TournamentState::Confirmation => {
-                        // We process only requests with
-                        // TournamentRequestState::ParticipationRequest|Declined
-                        match &request_state {
-                            TournamentRequestState::ConfirmationDeclined { reason } => {
-                                // Team did not confirm registration
-                                app.ui.push_log_event(
+                    TournamentState::Confirmation => match &request_state {
+                        TournamentRequestState::ConfirmationDeclined { reason } => {
+                            app.ui.push_log_event(
                                     Tick::now(),
                                     None,
-                                    format!(
-                                        "Team {team_id} did not confirm registration to tournament: {reason}."
-                                    ),
+                                    format!("Team {team_id} did not confirm registration to tournament: {reason}."),
                                     log::Level::Warn,
                                 );
-                            }
-                            TournamentRequestState::ParticipationRequest => {
-                                let (team, players) = if let Some(data) = team_data.as_ref() {
-                                    data
-                                } else {
-                                    let reason = "missing team data".to_string();
-                                    app.network_handler.send_tournament_registration_request(
+                        }
+                        TournamentRequestState::ParticipationRequest => {
+                            let (team, players) = if let Some(data) = team_data.as_ref() {
+                                data
+                            } else {
+                                send_and_log(
+                                        &mut app.network_handler,
+                                        &mut app.ui,
                                         tournament_id,
                                         team_id,
-                                        None,
                                         TournamentRequestState::ParticipationDeclined {
-                                            reason: reason.clone(),
+                                            reason: "missing team data".to_string(),
                                         },
+                                        format!("Participation to tournament {tournament_id} declined for team {team_id}: missing team data."),
+                                        log::Level::Warn,
                                     )?;
-                                    app.ui.push_log_event(Tick::now(), None, format!("Participation to tournament {tournament_id} declined for team {team_id}: {reason}."), log::Level::Warn);
-                                    return Ok(None);
-                                };
-                                match tournament.confirm_team_registration(
-                                    team,
-                                    players.clone(),
-                                    Tick::now(),
-                                ) {
-                                    Ok(()) => {
-                                        app.network_handler.send_tournament_registration_request(
+                                return Ok(None);
+                            };
+                            match tournament.confirm_team_registration(
+                                team,
+                                players.clone(),
+                                Tick::now(),
+                            ) {
+                                Ok(()) => {
+                                    send_and_log(
+                                            &mut app.network_handler,
+                                            &mut app.ui,
                                             tournament_id,
                                             team_id,
-                                            None,
                                             TournamentRequestState::ParticipationOk,
+                                            format!("Participation to tournament {tournament_id} confirmed for team {team_id}."),
+                                            log::Level::Info,
                                         )?;
-                                        app.ui.push_log_event(Tick::now(), None, format!("Participation to tournament {tournament_id} confirmed for team {team_id}."), log::Level::Info);
-                                    }
-                                    Err(err) => {
-                                        app.network_handler.send_tournament_registration_request(
+                                }
+                                Err(err) => {
+                                    send_and_log(
+                                            &mut app.network_handler,
+                                            &mut app.ui,
                                             tournament_id,
                                             team_id,
-                                            None,
                                             TournamentRequestState::ParticipationDeclined {
                                                 reason: err.to_string(),
                                             },
+                                            format!("Participation to tournament {tournament_id} declined for team {team_id}: {err}."),
+                                            log::Level::Warn,
                                         )?;
-                                        app.ui.push_log_event(Tick::now(), None, format!("Participation to tournament {tournament_id} declined for team {team_id}: {err}."), log::Level::Warn);
-                                    }
                                 }
                             }
-                            _ => discard_request_log_event(
-                                &mut app.ui,
-                                &request_state,
-                                &tournament.state(timestamp),
-                            ),
                         }
-                    }
+                        _ => discard_request_log_event(
+                            &mut app.ui,
+                            &request_state,
+                            &tournament.state(timestamp),
+                        ),
+                    },
 
                     _ => discard_request_log_event(
                         &mut app.ui,
@@ -388,11 +392,8 @@ impl NetworkCallback {
                     ),
                 }
             }
-            // Check if own_team is team_id (sender of request).
+            // ---- Own team: handle responses from organizer ----
             else if team_id == app.world.own_team_id {
-                // Only possible combinations are
-                // TournamentState::Registration && state = TournamentRegistrationState::(Registered || None)
-                // TournamentState::Confirmation  && state = TournamentRegistrationState::(Pending || Confirmed || None)
                 let own_team = app
                     .world
                     .teams
@@ -401,6 +402,26 @@ impl NetworkCallback {
 
                 let timestamp = Tick::now();
 
+                if let TournamentRequestState::Cancellation { reason } = &request_state {
+                    own_team.tournament_registration_state = TournamentRegistrationState::None;
+                    app.ui.push_popup(PopupMessage::Ok {
+                        message: format!(
+                            "{} tournament got cancelled: {}.",
+                            tournament.name(),
+                            reason
+                        ),
+                        timestamp: Tick::now(),
+                        is_skippable: true,
+                    });
+                    app.ui.push_log_event(
+                        Tick::now(),
+                        None,
+                        format!("Tournament {} cancelled: {}.", tournament_id, reason),
+                        log::Level::Warn,
+                    );
+                    return Ok(None);
+                }
+
                 match tournament.state(timestamp) {
                     TournamentState::Registration => {
                         if !matches!(
@@ -415,7 +436,6 @@ impl NetworkCallback {
                             return Ok(None);
                         }
 
-                        // We process only requests with state = TournamentRequestState::RegistrationOk|Declined
                         match &request_state {
                             TournamentRequestState::RegistrationDeclined { reason } => {
                                 app.ui.push_log_event(
@@ -427,7 +447,6 @@ impl NetworkCallback {
                                 own_team.tournament_registration_state =
                                     TournamentRegistrationState::None;
                             }
-
                             TournamentRequestState::RegistrationOk => {
                                 app.ui.push_log_event(
                                     Tick::now(),
@@ -438,7 +457,6 @@ impl NetworkCallback {
                                 own_team.tournament_registration_state =
                                     TournamentRegistrationState::Registered { tournament_id };
                             }
-
                             _ => discard_request_log_event(
                                 &mut app.ui,
                                 &request_state,
@@ -448,7 +466,6 @@ impl NetworkCallback {
                     }
 
                     TournamentState::Confirmation => {
-                        //If we are already Confirmed, return.
                         if matches!(
                             own_team.tournament_registration_state,
                             TournamentRegistrationState::Confirmed {tournament_id: id} if id == tournament_id
@@ -462,14 +479,12 @@ impl NetworkCallback {
                             return Ok(None);
                         }
 
-                        // Basically if tournament_registration_state == None | Pending
                         if !matches!(
                             own_team.tournament_registration_state,
                             TournamentRegistrationState::Registered {tournament_id: id} if id == tournament_id
                         ) {
                             own_team.tournament_registration_state =
                                 TournamentRegistrationState::None;
-
                             discard_request_log_event(
                                 &mut app.ui,
                                 &request_state,
@@ -479,30 +494,24 @@ impl NetworkCallback {
                         }
 
                         // Here we know that tournament_registration_state == Registered
-                        // We process only requests with TournamentRequestState::(ConfirmationRequest || ParticipationOk|Declined)
                         match &request_state {
-                            // We are asked to confirm our participation.
                             TournamentRequestState::ConfirmationRequest => {
-                                // First, check if we can participate.
                                 if let Err(err) = own_team
                                     .can_confirm_tournament_registration(tournament, Tick::now())
                                 {
-                                    app.network_handler.send_tournament_registration_request(
+                                    send_and_log(
+                                        &mut app.network_handler,
+                                        &mut app.ui,
                                         tournament_id,
                                         own_team.id,
-                                        None,
                                         TournamentRequestState::ConfirmationDeclined {
                                             reason: err.to_string(),
                                         },
+                                        format!("Cannot confirm tournament participation: {err}."),
+                                        log::Level::Warn,
                                     )?;
                                     own_team.tournament_registration_state =
                                         TournamentRegistrationState::None;
-                                    app.ui.push_log_event(
-                                        Tick::now(),
-                                        None,
-                                        format!("Cannot confirm tournament participation: {err}."),
-                                        log::Level::Warn,
-                                    );
                                     return Ok(None);
                                 }
 
@@ -526,8 +535,6 @@ impl NetworkCallback {
                                     log::Level::Info,
                                 )
                             }
-
-                            // Ourparticipation was declined :(
                             TournamentRequestState::ParticipationDeclined { reason } => {
                                 own_team.tournament_registration_state =
                                     TournamentRegistrationState::None;
@@ -539,7 +546,6 @@ impl NetworkCallback {
                                 );
                             }
                             TournamentRequestState::ParticipationOk => {
-                                // Our participation was confirmed!
                                 own_team.tournament_registration_state =
                                     TournamentRegistrationState::Confirmed { tournament_id };
                                 app.ui.push_log_event(
@@ -588,7 +594,7 @@ impl NetworkCallback {
                         (tournament.starting_at() - Tick::now()).formatted_as_time()
                     ),
                     is_skippable: true,
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
             }
 
@@ -608,14 +614,16 @@ impl NetworkCallback {
                 };
 
             if received_tournament_is_latest {
-                // We move tournament games to world to be able to siumulate them.
-                for game in tournament.games.iter() {
-                    if game.has_ended() && !app.world.past_games.contains_key(&game.id) {
-                        app.world
-                            .past_games
-                            .insert(game.id, GameSummary::from_game(game));
-                    } else if !game.has_ended() && !app.world.games.contains_key(&game.id) {
-                        app.world.games.insert(game.id, game.clone());
+                if !tournament.is_canceled() {
+                    // We move tournament games to world to be able to simulate them.
+                    for game in tournament.games.iter() {
+                        if game.has_ended() && !app.world.past_games.contains_key(&game.id) {
+                            app.world
+                                .past_games
+                                .insert(game.id, GameSummary::from_game(game));
+                        } else if !game.has_ended() && !app.world.games.contains_key(&game.id) {
+                            app.world.games.insert(game.id, game.clone());
+                        }
                     }
                 }
 
@@ -669,7 +677,7 @@ impl NetworkCallback {
                 app.ui.push_popup(PopupMessage::Ok {
                     message,
                     is_skippable: false,
-                    tick: timestamp,
+                    timestamp,
                 });
             }
 
@@ -684,6 +692,22 @@ impl NetworkCallback {
                 .swarm_panel
                 .update_player_ranking(&seed_info.player_ranking);
 
+            if !seed_info.chat_history.is_empty() {
+                let entries: Vec<_> = seed_info
+                    .chat_history
+                    .iter()
+                    .map(|e| {
+                        (
+                            e.timestamp,
+                            e.from_peer_id,
+                            e.author.clone(),
+                            e.message.clone(),
+                        )
+                    })
+                    .collect();
+                app.ui.push_chat_history(entries);
+            }
+
             app.world.dirty_network = true;
             Ok(None)
         })
@@ -749,7 +773,7 @@ impl NetworkCallback {
                             &trade
                                 .target_player
                                 .team
-                                .ok_or(anyhow!("Player in trade should have a team"))?,
+                                .ok_or_else(|| anyhow!("Player in trade should have a team"))?,
                         )?;
 
                         own_team.can_trade_players(
@@ -774,7 +798,7 @@ impl NetworkCallback {
                         app.ui.push_popup(PopupMessage::Ok {
                             message: "Trade accepted, players swapped.".to_string(),
                             is_skippable: false,
-                            tick: Tick::now(),
+                            timestamp: Tick::now(),
                         });
                         trade.state = NetworkRequestState::Ack;
                         app.network_handler.send_trade(trade)?;
@@ -836,7 +860,7 @@ impl NetworkCallback {
                             &trade
                                 .proposer_player
                                 .team
-                                .ok_or(anyhow!("Player in trade should have a team"))?,
+                                .ok_or_else(|| anyhow!("Player in trade should have a team"))?,
                         )?;
 
                         proposer_team.can_trade_players(
@@ -861,7 +885,7 @@ impl NetworkCallback {
                         app.ui.push_popup(PopupMessage::Ok {
                             message: "Trade accepted, players swapped.".to_string(),
                             is_skippable: false,
-                            tick: Tick::now(),
+                            timestamp: Tick::now(),
                         });
                         Ok(())
                     };
@@ -888,7 +912,7 @@ impl NetworkCallback {
 
                     app.ui.push_popup(PopupMessage::Error {
                         message: format!("Trade failed: {error_message}"),
-                        tick: Tick::now(),
+                        timestamp: Tick::now(),
                     });
 
                     return Err(anyhow!(format!("Trade failed: {error_message}")))?;
@@ -1026,7 +1050,7 @@ impl NetworkCallback {
                         app.ui.push_popup(PopupMessage::Ok {
                             message: "Challenge accepted, game is starting.".to_string(),
                             is_skippable: false,
-                            tick: Tick::now(),
+                            timestamp: Tick::now(),
                         });
 
                         app.network_handler.send_challenge(challenge)?;
@@ -1104,7 +1128,7 @@ impl NetworkCallback {
                         app.ui.push_popup(PopupMessage::Ok {
                             message: "Challenge accepted, game is starting.".to_string(),
                             is_skippable: false,
-                            tick: Tick::now(),
+                            timestamp: Tick::now(),
                         });
                         Ok(())
                     };
@@ -1117,7 +1141,7 @@ impl NetworkCallback {
                         app.network_handler.send_challenge(challenge)?;
                         app.ui.push_popup(PopupMessage::Error {
                             message: format!("Challenge failed: {err}"),
-                            tick: Tick::now(),
+                            timestamp: Tick::now(),
                         });
 
                         return Err(anyhow!(err.to_string()));
@@ -1139,7 +1163,7 @@ impl NetworkCallback {
 
                     app.ui.push_popup(PopupMessage::Error {
                         message: format!("Challenge failed: {error_message}"),
-                        tick: Tick::now(),
+                        timestamp: Tick::now(),
                     });
 
                     return Err(anyhow!("Challenge failed: {error_message}"))?;
@@ -1152,11 +1176,6 @@ impl NetworkCallback {
 
     pub fn call(&self, app: &mut App) -> AppResult<Option<String>> {
         match self {
-            Self::PushSwarmPanelChat {
-                timestamp,
-                peer_id,
-                text,
-            } => Self::push_swarm_panel_message(*timestamp, *peer_id, text.clone())(app),
             Self::PushSwarmPanelLog {
                 timestamp,
                 peer_id,
@@ -1173,6 +1192,9 @@ impl NetworkCallback {
             Self::CloseConnection { peer_id } => Self::close_connection(*peer_id)(app),
             Self::HandleConnectionEstablished { peer_id } => {
                 app.network_handler.send_own_team(&app.world)?;
+                app.ui
+                    .swarm_panel
+                    .add_peer_id(*app.network_handler.own_peer_id(), app.world.own_team_id);
 
                 app.ui.push_log_event(
                     Tick::now(),
@@ -1190,9 +1212,12 @@ impl NetworkCallback {
                     NetworkData::Team { timestamp, team } => {
                         Self::handle_team_topic(peer_id, timestamp, team)(app)
                     }
-                    NetworkData::Message { timestamp, message } => {
-                        Self::handle_message_topic(peer_id, timestamp, message)(app)
-                    }
+                    NetworkData::Message {
+                        timestamp,
+                        from_peer_id,
+                        author,
+                        message,
+                    } => Self::handle_message_topic(from_peer_id, timestamp, author, message)(app),
                     NetworkData::Challenge {
                         timestamp,
                         challenge,
