@@ -135,7 +135,9 @@ impl AppChannel {
             return Err(anyhow!("pty has been already allocated"));
         };
 
-        let writer = SSHWriterProxy::new(id, session.handle());
+        let handle = session.handle();
+        let channel_id = id;
+        let writer = SSHWriterProxy::new(id, handle.clone());
 
         let username = self.username.clone();
         let network_port = self.network_port;
@@ -157,6 +159,10 @@ impl AppChannel {
             if let Err(e) = app.run(tui).await {
                 log::error!("Error running app: {e}")
             };
+
+            // Send EOF and close the channel so the client returns to its prompt.
+            let _ = handle.eof(channel_id).await;
+            let _ = handle.close(channel_id).await;
         });
 
         self.window_change_request(width, height).await?;
