@@ -1,5 +1,5 @@
 #[cfg(feature = "relayer")]
-use crate::network::types::{PlayerRanking, TeamRanking};
+use crate::network::types::{ChatHistoryEntry, PlayerRanking, TeamRanking};
 use crate::{
     core::world::World,
     game_engine::{game::Game, Tournament, TournamentId},
@@ -31,6 +31,8 @@ static LEGACY_PERSISTED_GAMES_PREFIX: &str = "game_";
 static PERSISTED_TEAM_RANKING_FILENAME: &str = "relayer/team_ranking";
 #[cfg(feature = "relayer")]
 static PERSISTED_PLAYER_RANKING_FILENAME: &str = "relayer/player_ranking";
+#[cfg(feature = "relayer")]
+static PERSISTED_CHAT_HISTORY_FILENAME: &str = "relayer/chat_history";
 const COMPRESSION_LEVEL: u32 = 5;
 
 fn prefixed_world_filename(store_prefix: &str) -> String {
@@ -109,7 +111,7 @@ fn config_dirs() -> AppResult<PathBuf> {
     // Windows: C:\Users\Alice\AppData\Roaming\frittura\rebels
     // macOS:   /Users/Alice/Library/Application Support/org.frittura.rebels
     let dirs = directories::ProjectDirs::from("org", "frittura", "rebels")
-        .ok_or(anyhow!("Failed to get directories"))?;
+        .ok_or_else(|| anyhow!("Failed to get directories"))?;
     let config_dirs = dirs.config_dir().to_path_buf();
     if !config_dirs.exists() {
         std::fs::create_dir_all(&config_dirs)?;
@@ -250,6 +252,17 @@ pub fn load_player_ranking() -> AppResult<HashMap<PlayerId, PlayerRanking>> {
     load_from_json::<HashMap<PlayerId, PlayerRanking>>(PERSISTED_PLAYER_RANKING_FILENAME)
 }
 
+#[cfg(feature = "relayer")]
+pub fn save_chat_history(history: &[ChatHistoryEntry]) -> AppResult<()> {
+    save_to_json(PERSISTED_CHAT_HISTORY_FILENAME, &history)?;
+    Ok(())
+}
+
+#[cfg(feature = "relayer")]
+pub fn load_chat_history() -> AppResult<Vec<ChatHistoryEntry>> {
+    load_from_json::<Vec<ChatHistoryEntry>>(PERSISTED_CHAT_HISTORY_FILENAME)
+}
+
 pub fn get_world_size(store_prefix: &str) -> AppResult<u64> {
     let size = world_file_data(store_prefix)?.len();
     Ok(size)
@@ -257,7 +270,7 @@ pub fn get_world_size(store_prefix: &str) -> AppResult<u64> {
 
 pub fn reset_store() -> AppResult<()> {
     let dirs = directories::ProjectDirs::from("org", "frittura", "rebels")
-        .ok_or(anyhow!("Failed to get directories"))?;
+        .ok_or_else(|| anyhow!("Failed to get directories"))?;
     let config_dirs = dirs.config_dir();
     if config_dirs.exists() {
         std::fs::remove_dir_all(config_dirs)?;
@@ -311,11 +324,12 @@ pub fn world_file_data(store_prefix: &str) -> AppResult<std::fs::Metadata> {
 #[cfg(test)]
 mod tests {
     use crate::{
-        core::{player::Player, team::Team, world::World},
+        core::{player::Player, team::Team, world::World, MIN_PLAYERS_PER_GAME},
         types::{AppResult, PlayerMap},
     };
     use directories;
     use itertools::Itertools;
+    use libp2p::PeerId;
     use std::fs::File;
 
     #[test]
@@ -354,6 +368,8 @@ mod tests {
         use crate::network::types::{NetworkData, NetworkTeam};
         let value = NetworkData::Message {
             timestamp: 0,
+            from_peer_id: PeerId::random(),
+            author: "Test".to_string(),
             message: "Hello".to_string(),
         };
         let serialized_data = serialize(&value)?;
@@ -363,7 +379,7 @@ mod tests {
         let mut team = Team::random(None);
 
         let mut players = PlayerMap::new();
-        for _ in 0..5 {
+        for _ in 0..MIN_PLAYERS_PER_GAME {
             let player = Player::default().randomize(None);
             team.player_ids.push(player.id);
             players.insert(player.id, player);
