@@ -30,8 +30,9 @@ use crate::{
     ui::constants::*,
 };
 use core::fmt::Debug;
-use crossterm::event::KeyCode;
 use itertools::Itertools;
+use ratatui::crossterm;
+use ratatui::crossterm::event::KeyCode;
 use ratatui::style::{Style, Stylize};
 use ratatui::{
     layout::{Constraint, Layout, Margin},
@@ -65,11 +66,11 @@ impl GamePanel {
         Self::default()
     }
 
-    pub fn toggle_pitch_view(&mut self) {
+    pub const fn toggle_pitch_view(&mut self) {
         self.pitch_view = !self.pitch_view;
     }
 
-    pub fn toggle_player_status_view(&mut self) {
+    pub const fn toggle_player_status_view(&mut self) {
         self.player_status_view = !self.player_status_view;
     }
 
@@ -157,7 +158,7 @@ impl GamePanel {
         Ok(())
     }
 
-    fn build_game_list(&mut self, frame: &mut UiFrame, world: &World, area: Rect) {
+    fn build_game_list(&self, frame: &mut UiFrame, world: &World, area: Rect) {
         let maybe_options = self
             .game_ids
             .iter()
@@ -199,7 +200,7 @@ impl GamePanel {
         }
     }
 
-    fn build_recent_games_list(&mut self, frame: &mut UiFrame, world: &World, area: Rect) {
+    fn build_recent_games_list(&self, frame: &mut UiFrame, world: &World, area: Rect) {
         let maybe_options = self
             .recent_game_ids
             .iter()
@@ -256,7 +257,7 @@ impl GamePanel {
         }
     }
 
-    fn build_game_buttons(&mut self, frame: &mut UiFrame, area: Rect) {
+    fn build_game_buttons(&self, frame: &mut UiFrame, area: Rect) {
         if self.index.is_none() {
             return;
         };
@@ -605,13 +606,8 @@ impl GamePanel {
         Ok(())
     }
 
-    fn build_bottom_panel(
-        &mut self,
-        frame: &mut UiFrame,
-        world: &World,
-        area: Rect,
-    ) -> AppResult<()> {
-        let split = Layout::horizontal([Constraint::Min(8), Constraint::Length(73)]).split(area);
+    fn build_bottom_panel(&self, frame: &mut UiFrame, world: &World, area: Rect) -> AppResult<()> {
+        let split = Layout::horizontal([Constraint::Fill(1), Constraint::Length(73)]).split(area);
 
         let game = if let Some(game) = Self::selected_game(
             world,
@@ -1003,12 +999,8 @@ impl GamePanel {
         let box_area = Layout::vertical([
             Constraint::Ratio(1, 2),
             Constraint::Ratio(1, 2),
-            Constraint::Min(0),
         ])
-        .split(area.inner(Margin {
-            horizontal: 1,
-            vertical: 0,
-        }));
+        .split(area);
 
         let home_box_split = Layout::vertical([Constraint::Min(0), Constraint::Length(1)])
             .split(box_area[0].inner(Margin::new(1, 1)));
@@ -1092,7 +1084,7 @@ impl GamePanel {
             Constraint::Length(3),                          //blocks
             Constraint::Length(3),                          //brawls
             Constraint::Length(3),                          //plus minus
-            Constraint::Min(0),
+            Constraint::Fill(1),
         ];
 
         let home_table = Self::build_stats_table(&game.home_team_in_game.stats, home_players)
@@ -1114,12 +1106,8 @@ impl GamePanel {
         let box_area = Layout::vertical([
             Constraint::Ratio(1, 2),
             Constraint::Ratio(1, 2),
-            Constraint::Min(0),
         ])
-        .split(area.inner(Margin {
-            horizontal: 1,
-            vertical: 0,
-        }));
+        .split(area);
 
         frame.render_widget(home_table.block(default_block()), box_area[0]);
         frame.render_widget(away_table.block(default_block()), box_area[1]);
