@@ -22,8 +22,9 @@ use crate::{
 };
 use anyhow::anyhow;
 use core::fmt::Debug;
-use crossterm::event::KeyCode;
 use itertools::Itertools;
+use ratatui::crossterm;
+use ratatui::crossterm::event::KeyCode;
 use ratatui::style::Stylize;
 use ratatui::{
     layout::Margin,
@@ -46,14 +47,14 @@ pub enum MyTeamView {
 }
 
 impl MyTeamView {
-    fn next(&self) -> Self {
+    const fn next(&self) -> Self {
         match self {
-            MyTeamView::Info => MyTeamView::Team,
-            MyTeamView::Team => MyTeamView::Games,
-            MyTeamView::Games => MyTeamView::Market,
-            MyTeamView::Market => MyTeamView::Shipyard,
-            MyTeamView::Shipyard => MyTeamView::Asteroids,
-            MyTeamView::Asteroids => MyTeamView::Info,
+            Self::Info => Self::Team,
+            Self::Team => Self::Games,
+            Self::Games => Self::Market,
+            Self::Market => Self::Shipyard,
+            Self::Shipyard => Self::Asteroids,
+            Self::Asteroids => Self::Info,
         }
     }
 }
@@ -85,7 +86,7 @@ pub struct MyTeamPanel {
     current_planet_id: Option<PlanetId>,
     tick: usize,
     gif_map: GifMap,
-    // players_table: ClickableTable<'static>,
+    players_table: ClickableTable<'static>,
 }
 
 impl MyTeamPanel {
@@ -1015,7 +1016,7 @@ impl MyTeamPanel {
             let game_summary = world
                 .past_games
                 .get(&game_id)
-                .ok_or(anyhow!("Unable to get past game."))?;
+                .ok_or_else(|| anyhow!("Unable to get past game."))?;
 
             let mut lines = vec![
                 Line::from(format!(
@@ -1393,7 +1394,7 @@ impl MyTeamPanel {
             own_team,
             frame,
             v_split[1],
-        )?;
+        );
         self.render_upgrade_spaceship_button(possible_upgrade, own_team, frame, v_split[2])?;
 
         Ok(())
@@ -1609,7 +1610,7 @@ impl MyTeamPanel {
                 own_team,
                 frame,
                 v_split[2],
-            )?;
+            );
         }
 
         Ok(())
@@ -1704,7 +1705,7 @@ impl MyTeamPanel {
         let popup_message = PopupMessage::AbandonAsteroid {
             asteroid_name: asteroid.name.clone(),
             asteroid_id,
-            tick: Tick::now(),
+            timestamp: Tick::now(),
         };
 
         let abandon_asteroid_button =
@@ -1851,7 +1852,7 @@ impl MyTeamPanel {
             player_name: player.info.full_name(),
             player_id,
             not_enough_players_for_game: own_team.player_ids.len() - 1 < MIN_PLAYERS_PER_GAME,
-            tick: Tick::now(),
+            timestamp: Tick::now(),
         };
         let mut release_button = Button::new(
             format!("Fire {}", player.info.short_name()),
@@ -1878,7 +1879,7 @@ impl MyTeamPanel {
         players: &'a Vec<&Player>,
         world: &'a World,
         table_width: u16,
-    ) -> AppResult<ClickableTable<'a>> {
+    ) -> AppResult<ClickableTable<'static>> {
         let own_team = world.get_own_team()?;
         let header_cells = [
             "Name",
@@ -2040,22 +2041,17 @@ impl MyTeamPanel {
             .sort_by_rating();
 
         let player_index = if let Some(index) = self.player_index {
-            index.min(sorted_players.len() - 1)
+            index.min(own_team.player_ids.len() - 1)
         } else {
             return Ok(());
         };
         let player = sorted_players[player_index];
 
         let top_split =
-            Layout::horizontal([Constraint::Min(10), Constraint::Length(60)]).split(area);
+            Layout::horizontal([Constraint::Fill(1), Constraint::Length(60)]).split(area);
 
-        let table = Self::build_players_table(&sorted_players, world, top_split[0].width)?;
         frame.render_stateful_interactive_widget(
-            table.block(default_block().title(format!(
-                "{} {} ↓/↑",
-                own_team.name.clone(),
-                world.team_rating(&own_team.id).unwrap_or_default().stars()
-            ))),
+            &self.players_table,
             top_split[0],
             &mut ClickableTableState::default().with_selected(self.player_index),
         );
@@ -2101,8 +2097,8 @@ impl MyTeamPanel {
                     vec![
                         Line::from("Currently playing".to_string()).centered(),
                         Line::default(),
-                        Line::from(game_text.to_string()).centered(),
-                        Line::from(game.timer.format().to_string()).centered(),
+                        Line::from(game_text).centered(),
+                        Line::from(game.timer.format()).centered(),
                     ],
                     UiCallback::GoToGame { game_id },
                 )
@@ -2209,8 +2205,8 @@ impl MyTeamPanel {
 
         let explore_split =
             Layout::horizontal([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)]).split(split[1]);
-        if let Ok(explore_button) = space_adventure_button(world, own_team) {
-            frame.render_interactive_widget(explore_button, explore_split[0]);
+        if let Ok(space_adventure_button) = space_adventure_button(world, own_team) {
+            frame.render_interactive_widget(space_adventure_button, explore_split[0]);
         }
         if let Ok(explore_button) = explore_button(world, own_team) {
             frame.render_interactive_widget(explore_button, explore_split[1]);
@@ -2373,11 +2369,11 @@ impl MyTeamPanel {
         Ok(())
     }
 
-    pub fn set_view(&mut self, view: MyTeamView) {
+    pub const fn set_view(&mut self, view: MyTeamView) {
         self.view = view;
     }
 
-    pub fn reset_view(&mut self) {
+    pub const fn reset_view(&mut self) {
         self.set_view(MyTeamView::Info);
     }
 }
@@ -2476,7 +2472,20 @@ impl Screen for MyTeamPanel {
                     .unwrap()
             });
 
-            // self.players_table = Self::build_players_table(players, world, table_width)
+            let sorted_players = own_team
+                .player_ids
+                .iter()
+                .map(|id| world.players.get(id).unwrap())
+                .collect_vec()
+                .sort_by_rating();
+
+            let table_width = UI_SCREEN_SIZE.0 - 60;
+            self.players_table = Self::build_players_table(&sorted_players, world, table_width)?
+                .block(default_block().title(format!(
+                    "{} {} ↓/↑",
+                    own_team.name,
+                    world.team_rating(&own_team.id).unwrap_or_default().stars()
+                )));
         }
 
         self.game_index = if !self.past_game_ids.is_empty() {
