@@ -23,11 +23,11 @@ use crate::{
 };
 use core::fmt::Debug;
 use core::panic;
-use crossterm::event::{KeyCode, KeyEvent};
 use itertools::Itertools;
 use rand::seq::SliceRandom;
 use rand::SeedableRng;
 use rand_chacha::ChaCha8Rng;
+use ratatui::crossterm::event::{KeyCode, KeyEvent};
 use ratatui::style::Styled;
 use ratatui::text::Line;
 use ratatui::{
@@ -40,6 +40,7 @@ use std::cmp::min;
 use std::collections::HashMap;
 use strum::IntoEnumIterator;
 use tui_textarea::{CursorMove, TextArea};
+use uuid::uuid;
 
 const INITIAL_TEAM_SIZE: usize = 5;
 const SPACESHIP_MODELS: [SpaceshipPrefab; 3] = [
@@ -47,6 +48,12 @@ const SPACESHIP_MODELS: [SpaceshipPrefab; 3] = [
     SpaceshipPrefab::Orwell,
     SpaceshipPrefab::Ibarruri,
 ];
+const INITIAL_PLANET_IDS: [PlanetId; 4] = [
+    uuid!("71a43700-0000-0000-0002-000000000000"), //Earth
+    uuid!("71a43700-0000-0000-0003-000000000000"), //Moon
+    uuid!("71a43700-0000-0000-0003-000000000004"), //Mizu
+    uuid!("71a43700-0000-0000-0003-000000000003"), //Frittura
+];
 
 #[derive(Debug, Default, PartialOrd, PartialEq)]
 pub enum CreationState {
@@ -61,27 +68,27 @@ pub enum CreationState {
 }
 
 impl CreationState {
-    pub fn next(&self) -> Self {
+    pub const fn next(&self) -> Self {
         match self {
-            CreationState::TeamName => CreationState::ShipName,
-            CreationState::ShipName => CreationState::Planet,
-            CreationState::Planet => CreationState::Jersey,
-            CreationState::Jersey => CreationState::ShipModel,
-            CreationState::ShipModel => CreationState::Players,
-            CreationState::Players => CreationState::Done,
-            CreationState::Done => CreationState::Done,
+            Self::TeamName => Self::ShipName,
+            Self::ShipName => Self::Planet,
+            Self::Planet => Self::Jersey,
+            Self::Jersey => Self::ShipModel,
+            Self::ShipModel => Self::Players,
+            Self::Players => Self::Done,
+            Self::Done => Self::Done,
         }
     }
 
-    pub fn previous(&self) -> Self {
+    pub const fn previous(&self) -> Self {
         match self {
-            CreationState::TeamName => CreationState::TeamName,
-            CreationState::ShipName => CreationState::TeamName,
-            CreationState::Planet => CreationState::ShipName,
-            CreationState::Jersey => CreationState::Planet,
-            CreationState::ShipModel => CreationState::Jersey,
-            CreationState::Players => CreationState::ShipModel,
-            CreationState::Done => CreationState::Players,
+            Self::TeamName => Self::TeamName,
+            Self::ShipName => Self::TeamName,
+            Self::Planet => Self::ShipName,
+            Self::Jersey => Self::Planet,
+            Self::ShipModel => Self::Jersey,
+            Self::Players => Self::ShipModel,
+            Self::Done => Self::Players,
         }
     }
 }
@@ -186,7 +193,7 @@ impl NewTeamScreen {
         self.set_index(0);
     }
 
-    fn render_intro(&mut self, frame: &mut UiFrame, area: Rect) {
+    fn render_intro(&self, frame: &mut UiFrame, area: Rect) {
         let text = "
         It's the year 2101. Corporations have taken over the world. 
         The only way to be free is to join a pirate crew and start plundering the galaxy.
@@ -208,7 +215,7 @@ impl NewTeamScreen {
         frame.render_widget(default_block(), area);
     }
 
-    fn render_spaceship(&mut self, frame: &mut UiFrame, area: Rect) {
+    fn render_spaceship(&self, frame: &mut UiFrame, area: Rect) {
         let split = Layout::horizontal([
             Constraint::Length(SPACESHIP_IMAGE_WIDTH as u16 + 2),
             Constraint::Min(1),
@@ -312,7 +319,7 @@ impl NewTeamScreen {
         }
     }
 
-    fn render_jersey_selection(&mut self, frame: &mut UiFrame, area: Rect) {
+    fn render_jersey_selection(&self, frame: &mut UiFrame, area: Rect) {
         if self.state > CreationState::Jersey {
             let selected_jersey_style = self.jersey_styles[self.jersey_style_index];
             frame.render_widget(
@@ -469,7 +476,7 @@ impl NewTeamScreen {
     }
 
     fn render_planet_selection(
-        &mut self,
+        &self,
         frame: &mut UiFrame,
         world: &World,
         area: Rect,
@@ -585,7 +592,7 @@ impl NewTeamScreen {
         INITIAL_TEAM_BALANCE as i32 - hiring_costs - ship_cost as i32
     }
 
-    fn render_remaining_balance(&mut self, frame: &mut UiFrame, area: Rect) {
+    fn render_remaining_balance(&self, frame: &mut UiFrame, area: Rect) {
         let remaining_balance = self.get_remaining_balance();
         let text = format!(" Remaining balance: {remaining_balance:>} sat");
 
@@ -597,12 +604,7 @@ impl NewTeamScreen {
         frame.render_widget(Paragraph::new(text).block(block), area);
     }
 
-    fn render_player_list(
-        &mut self,
-        frame: &mut UiFrame,
-        world: &World,
-        area: Rect,
-    ) -> AppResult<()> {
+    fn render_player_list(&self, frame: &mut UiFrame, world: &World, area: Rect) -> AppResult<()> {
         if self.state < CreationState::Players {
             frame.render_widget(
                 default_block()
@@ -622,12 +624,13 @@ impl NewTeamScreen {
             .iter()
             .map(|&player_data| {
                 let player_id = player_data.0;
-                let mut style = UiStyle::DEFAULT;
-                if self.selected_players.contains(&player_id)
+                let style = if self.selected_players.contains(&player_id)
                     && self.state <= CreationState::Players
                 {
-                    style = UiStyle::OK;
-                }
+                    UiStyle::OK
+                } else {
+                    UiStyle::DEFAULT
+                };
 
                 if self.state > CreationState::Players
                     && !self.selected_players.contains(&player_id)
@@ -696,12 +699,7 @@ impl NewTeamScreen {
         Ok(())
     }
 
-    fn render_confirm_box(
-        &mut self,
-        frame: &mut UiFrame,
-        world: &World,
-        area: Rect,
-    ) -> AppResult<()> {
+    fn render_confirm_box(&self, frame: &mut UiFrame, world: &World, area: Rect) -> AppResult<()> {
         let split = Layout::vertical([
             Constraint::Min(1),
             Constraint::Length(4),
@@ -781,15 +779,11 @@ impl Screen for NewTeamScreen {
             self.planet_ids = world
                 .planets
                 .keys()
-                .filter(|&planet_id| {
-                    if let Some(planet) = world.planets.get(planet_id) {
-                        return planet.total_population() > 0;
-                    }
-                    false
-                })
+                .filter(|planet_id| INITIAL_PLANET_IDS.contains(planet_id))
                 .sorted_by(|a, b| a.cmp(b))
                 .copied()
                 .collect_vec();
+
             for player in world.players.values() {
                 if player.team.is_none() {
                     let planet_players = self
@@ -1139,7 +1133,7 @@ impl Screen for NewTeamScreen {
                                 jersey_style: self.jersey_styles[self.jersey_style_index],
                                 jersey_colors: self.get_team_colors(),
                                 players: self.selected_players.clone(),
-                                spaceship: self.selected_ship().clone(),
+                                spaceship: self.selected_ship(),
                             });
                         }
                         KeyCode::Backspace => {
