@@ -6,6 +6,7 @@ use super::ui_frame::UiFrame;
 use super::utils::{img_to_lines, input_from_key_event, validate_textarea_input};
 use super::widgets::{default_block, thick_block};
 use crate::core::planet::PlanetType;
+use crate::core::MAX_SKILL;
 use crate::core::{player::Player, resources::Resource, skill::Rated};
 use crate::image::utils::open_gif;
 use crate::types::*;
@@ -16,6 +17,7 @@ use crate::ui::ui_key;
 use anyhow::anyhow;
 use core::fmt::Debug;
 use itertools::Itertools;
+use ratatui::crossterm;
 use ratatui::layout::{Constraint, Layout};
 use ratatui::layout::{Margin, Rect};
 use ratatui::style::Stylize;
@@ -31,75 +33,84 @@ const TREASURE_GIF_ANIMATION_DELAY: Tick = 450;
 pub enum PopupMessage {
     Error {
         message: String,
-        tick: Tick,
+        timestamp: Tick,
+    },
+    Warning {
+        message: String,
+        timestamp: Tick,
     },
     Ok {
         message: String,
         is_skippable: bool,
-        tick: Tick,
+        timestamp: Tick,
     },
     PromptQuit {
         during_space_adventure: bool,
-        tick: Tick,
+        timestamp: Tick,
     },
     ReleasePlayer {
         player_name: String,
         player_id: PlayerId,
         not_enough_players_for_game: bool,
-        tick: Tick,
+        timestamp: Tick,
+    },
+    ConfirmSpaceAdventure {
+        has_shooter: bool,
+        average_tiredness: f32,
+        timestamp: Tick,
     },
     AbandonAsteroid {
         asteroid_name: String,
         asteroid_id: PlanetId,
-        tick: Tick,
+        timestamp: Tick,
     },
     AsteroidNameDialog {
-        tick: Tick,
+        timestamp: Tick,
         asteroid_type: usize,
     },
     BuildSpaceCove {
         asteroid_name: String,
         asteroid_id: PlanetId,
-        tick: Tick,
+        timestamp: Tick,
     },
     PortalFound {
         player_name: String,
         portal_target: String,
-        tick: Tick,
+        timestamp: Tick,
     },
     ExplorationResult {
         planet_name: String,
         resources: ResourceMap,
         players: Vec<Player>,
-        tick: Tick,
+        timestamp: Tick,
     },
     TeamLanded {
         team_name: String,
         planet_name: String,
         planet_filename: String,
         planet_type: PlanetType,
-        tick: Tick,
+        timestamp: Tick,
     },
     Tutorial {
         index: usize,
-        tick: Tick,
+        timestamp: Tick,
     },
 }
 
 impl PopupMessage {
-    const MAX_TUTORIAL_PAGE: usize = 4;
+    const MAX_TUTORIAL_PAGE: usize = 7;
     fn rect(&self, area: Rect) -> Rect {
         let (width, height) = match self {
-            PopupMessage::AsteroidNameDialog { .. } => (54, 28),
-            PopupMessage::PortalFound { .. } => (54, 44),
-            PopupMessage::ExplorationResult { resources, .. } => {
+            Self::AsteroidNameDialog { .. } => (54, 28),
+            Self::PortalFound { .. } => (54, 44),
+            Self::ExplorationResult { resources, .. } => {
                 if resources.value(&Resource::GOLD) > 0 {
                     (54, 26)
                 } else {
                     (54, 16)
                 }
             }
-            PopupMessage::TeamLanded { .. } => (54, 26),
+            Self::TeamLanded { .. } => (54, 26),
             _ => (48, 16),
         };
 
@@ -130,10 +141,10 @@ impl PopupMessage {
         Rect::new(x, y, rect_width, rect_height)
     }
 
-    pub fn is_skippable(&self) -> bool {
+    pub const fn is_skippable(&self) -> bool {
         match self {
-            PopupMessage::Error { .. } => true,
-            PopupMessage::Ok { is_skippable, .. } => *is_skippable,
+            Self::Error { .. } | Self::Warning { .. } => true,
+            Self::Ok { is_skippable, .. } => *is_skippable,
             _ => false,
         }
     }
@@ -146,7 +157,7 @@ impl PopupMessage {
         key_event: crossterm::event::KeyEvent,
     ) -> Option<UiCallback> {
         match self {
-            PopupMessage::AsteroidNameDialog { tick, .. } => {
+            Self::AsteroidNameDialog { timestamp, .. } => {
                 if key_event.code == ui_key::YES_TO_DIALOG {
                     let mut name = popup_input.lines()[0].clone();
                     name = name
@@ -156,7 +167,7 @@ impl PopupMessage {
                         .take(MAX_NAME_LENGTH)
                         .collect();
                     if validate_textarea_input(popup_input, "Asteroid name") {
-                        let filename = format!("asteroid{}", tick % 30);
+                        let filename = format!("asteroid{}", timestamp % 30);
                         return Some(UiCallback::NameAndAcceptAsteroid { name, filename });
                     }
                 } else if key_event.code == ui_key::NO_TO_DIALOG {
@@ -169,7 +180,7 @@ impl PopupMessage {
                 }
             }
 
-            PopupMessage::ReleasePlayer { player_id, .. } => {
+            Self::ReleasePlayer { player_id, .. } => {
                 if key_event.code == ui_key::YES_TO_DIALOG {
                     return Some(UiCallback::ReleasePlayer {
                         player_id: *player_id,
@@ -179,7 +190,15 @@ impl PopupMessage {
                 }
             }
 
-            PopupMessage::AbandonAsteroid { asteroid_id, .. } => {
+            Self::ConfirmSpaceAdventure { .. } => {
+                if key_event.code == ui_key::YES_TO_DIALOG {
+                    return Some(UiCallback::StartSpaceAdventure);
+                } else if key_event.code == ui_key::NO_TO_DIALOG {
+                    return Some(UiCallback::CloseUiPopup);
+                }
+            }
+
+            Self::AbandonAsteroid { asteroid_id, .. } => {
                 if key_event.code == ui_key::YES_TO_DIALOG {
                     return Some(UiCallback::AbandonAsteroid {
                         asteroid_id: *asteroid_id,
@@ -189,7 +208,7 @@ impl PopupMessage {
                 }
             }
 
-            PopupMessage::BuildSpaceCove { asteroid_id, .. } => {
+            Self::BuildSpaceCove { asteroid_id, .. } => {
                 if key_event.code == ui_key::YES_TO_DIALOG {
                     return Some(UiCallback::BuildSpaceCove {
                         asteroid_id: *asteroid_id,
@@ -199,7 +218,7 @@ impl PopupMessage {
                 }
             }
 
-            PopupMessage::PromptQuit { .. } => {
+            Self::PromptQuit { .. } => {
                 if key_event.code == ui_key::YES_TO_DIALOG {
                     return Some(UiCallback::QuitGame);
                 } else if key_event.code == ui_key::NO_TO_DIALOG {
@@ -207,17 +226,38 @@ impl PopupMessage {
                 }
             }
 
-            PopupMessage::Tutorial { index, .. } => {
-                if key_event.code == ui_key::YES_TO_DIALOG
-                    && *index == PopupMessage::MAX_TUTORIAL_PAGE
-                {
-                    return Some(UiCallback::CloseUiPopup);
-                } else if key_event.code == ui_key::YES_TO_DIALOG {
+            Self::Tutorial { index, .. } => match key_event.code {
+                ui_key::YES_TO_DIALOG => {
+                    if *index == Self::MAX_TUTORIAL_PAGE {
+                        return Some(UiCallback::CloseUiPopup);
+                    }
                     return Some(UiCallback::PushTutorialPage { index: index + 1 });
-                } else if key_event.code == ui_key::NO_TO_DIALOG {
+                }
+
+                ui_key::NO_TO_DIALOG => {
                     return Some(UiCallback::CloseUiPopup);
                 }
-            }
+
+                code => match index {
+                    2 if code == ui_key::GO_TO_CHALLENGES => {
+                        return Some(UiCallback::TutorialGoToChallenges)
+                    }
+                    3 if code == ui_key::GO_TO_MARKET => {
+                        return Some(UiCallback::TutorialGoToMarket)
+                    }
+                    4 if code == ui_key::GO_TO_SHIPYARD => {
+                        return Some(UiCallback::TutorialGoToShipyard)
+                    }
+                    5 if code == ui_key::GO_TO_FREE_PIRATES => {
+                        return Some(UiCallback::TutorialGoToFreePirates)
+                    }
+                    6 if code == ui_key::GO_TO_SPACE_ADVENTURE => {
+                        return Some(UiCallback::TutorialGoToSpaceAdventure)
+                    }
+                    7 if code == ui_key::GO_TO_CHAT => return Some(UiCallback::TutorialGoToChat),
+                    _ => {}
+                },
+            },
             _ => {
                 if key_event.code == ui_key::YES_TO_DIALOG || key_event.code == ui_key::NO_TO_DIALOG
                 {
@@ -248,12 +288,14 @@ impl PopupMessage {
         frame.render_widget(Clear, rect);
         frame.render_widget(thick_block(), rect);
         match self {
-            PopupMessage::Ok { message, tick, .. } => {
+            Self::Ok {
+                message, timestamp, ..
+            } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Message: {} {}",
-                        tick.formatted_as_date(),
-                        tick.formatted_as_time()
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::OK))
@@ -284,12 +326,12 @@ impl PopupMessage {
                 );
             }
 
-            PopupMessage::Error { message, tick } => {
+            Self::Error { message, timestamp } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Error: {} {}",
-                        tick.formatted_as_date(),
-                        tick.formatted_as_time()
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::ERROR))
@@ -320,7 +362,43 @@ impl PopupMessage {
                 );
             }
 
-            PopupMessage::ReleasePlayer {
+            Self::Warning { message, timestamp } => {
+                frame.render_widget(
+                    Paragraph::new(format!(
+                        "Warning: {} {}",
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
+                    ))
+                    .bold()
+                    .block(default_block().border_style(UiStyle::WARNING))
+                    .centered(),
+                    split[0],
+                );
+                frame.render_widget(
+                    Paragraph::new(message.clone())
+                        .centered()
+                        .wrap(Wrap { trim: true }),
+                    split[1].inner(Margin {
+                        horizontal: 1,
+                        vertical: 1,
+                    }),
+                );
+                let button = Button::new(UiText::YES, UiCallback::CloseUiPopup)
+                    .set_hover_text("Close the popup")
+                    .set_hotkey(ui_key::YES_TO_DIALOG)
+                    .block(default_block().border_style(UiStyle::OK))
+                    .set_layer(1);
+
+                frame.render_interactive_widget(
+                    button,
+                    split[2].inner(Margin {
+                        vertical: 0,
+                        horizontal: 8,
+                    }),
+                );
+            }
+
+            Self::ReleasePlayer {
                 player_name,
                 player_id,
                 not_enough_players_for_game,
@@ -329,21 +407,18 @@ impl PopupMessage {
                 frame.render_widget(
                     Paragraph::new("Attention!")
                         .bold()
-                        .block(default_block().border_style(UiStyle::HIGHLIGHT))
+                        .block(default_block().border_style(UiStyle::WARNING))
                         .centered(),
                     split[0],
                 );
-                let extra_warning = if *not_enough_players_for_game {
-                    "\n\nThere will be not enough players for games!"
-                } else {
-                    ""
-                };
+
+                let mut text =
+                    format!("Are you sure you want to release {player_name} from the crew?");
+                if *not_enough_players_for_game {
+                    text.push_str("\nThere will be not enough players for games!");
+                }
                 frame.render_widget(
-                    Paragraph::new(format!(
-                        "Are you sure you want to release {player_name} from the crew?{extra_warning}"
-                    ))
-                    .centered()
-                    .wrap(Wrap { trim: true }),
+                    Paragraph::new(text).centered().wrap(Wrap { trim: true }),
                     split[1].inner(Margin {
                         horizontal: 1,
                         vertical: 1,
@@ -376,7 +451,60 @@ impl PopupMessage {
                 frame.render_interactive_widget(no_button, buttons_split[1]);
             }
 
-            PopupMessage::AbandonAsteroid {
+            Self::ConfirmSpaceAdventure {
+                has_shooter,
+                average_tiredness,
+                ..
+            } => {
+                frame.render_widget(
+                    Paragraph::new("Attention!")
+                        .bold()
+                        .block(default_block().border_style(UiStyle::WARNING))
+                        .centered(),
+                    split[0],
+                );
+
+                let mut text = format!(
+                    "Go on a Space Adventure? It will spend 25% of your pirates' energy{}.",
+                    if *average_tiredness > MAX_SKILL / 2.0 {
+                        " and they are already quite tired"
+                    } else {
+                        ""
+                    }
+                );
+                if !has_shooter {
+                    text.push_str("Your spaceship has no shooters, it will be very dangerous!");
+                };
+                frame.render_widget(
+                    Paragraph::new(text).centered().wrap(Wrap { trim: true }),
+                    split[1].inner(Margin {
+                        horizontal: 1,
+                        vertical: 1,
+                    }),
+                );
+
+                let buttons_split =
+                    Layout::horizontal([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)])
+                        .split(split[2]);
+
+                let confirm_button = Button::new(UiText::YES, UiCallback::StartSpaceAdventure)
+                    .set_hover_text("Start space adventure")
+                    .set_hotkey(ui_key::YES_TO_DIALOG)
+                    .block(default_block().border_style(UiStyle::OK))
+                    .set_layer(1);
+
+                frame.render_interactive_widget(confirm_button, buttons_split[0]);
+
+                let no_button = Button::new(UiText::NO, UiCallback::CloseUiPopup)
+                    .set_hover_text("Don't start space adventure")
+                    .set_hotkey(ui_key::NO_TO_DIALOG)
+                    .block(default_block().border_style(UiStyle::ERROR))
+                    .set_layer(1);
+
+                frame.render_interactive_widget(no_button, buttons_split[1]);
+            }
+
+            Self::AbandonAsteroid {
                 asteroid_name,
                 asteroid_id,
                 ..
@@ -384,7 +512,7 @@ impl PopupMessage {
                 frame.render_widget(
                     Paragraph::new("Attention!")
                         .bold()
-                        .block(default_block().border_style(UiStyle::HIGHLIGHT))
+                        .block(default_block().border_style(UiStyle::WARNING))
                         .centered(),
                     split[0],
                 );
@@ -426,7 +554,7 @@ impl PopupMessage {
                 frame.render_interactive_widget(no_button, buttons_split[1]);
             }
 
-            PopupMessage::BuildSpaceCove {
+            Self::BuildSpaceCove {
                 asteroid_name,
                 asteroid_id,
                 ..
@@ -434,7 +562,7 @@ impl PopupMessage {
                 frame.render_widget(
                     Paragraph::new("Attention!")
                         .bold()
-                        .block(default_block().border_style(UiStyle::HIGHLIGHT))
+                        .block(default_block().border_style(UiStyle::WARNING))
                         .centered(),
                     split[0],
                 );
@@ -476,20 +604,23 @@ impl PopupMessage {
                 frame.render_interactive_widget(no_button, buttons_split[1]);
             }
 
-            PopupMessage::PromptQuit {
+            Self::PromptQuit {
                 during_space_adventure,
                 ..
             } => {
                 frame.render_widget(
                     Paragraph::new("Attention!")
                         .bold()
-                        .block(default_block().border_style(UiStyle::HIGHLIGHT))
+                        .block(default_block().border_style(UiStyle::WARNING))
                         .centered(),
                     split[0],
                 );
 
                 let text = if *during_space_adventure {
-                    format!("Are you sure you want to quit?\nYou will lose the whole cargo! Go back to the base first\n(Press '{}')", ui_key::space::BACK_TO_BASE)
+                    format!(
+                        "Are you sure you want to quit?\nTo go back to the base press '{}'",
+                        ui_key::space::BACK_TO_BASE
+                    )
                 } else {
                     "Are you sure you want to quit?".to_string()
                 };
@@ -522,15 +653,15 @@ impl PopupMessage {
                 frame.render_interactive_widget(no_button, buttons_split[1]);
             }
 
-            PopupMessage::AsteroidNameDialog {
-                tick,
+            Self::AsteroidNameDialog {
+                timestamp,
                 asteroid_type,
             } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Asteroid discovered: {} {}",
-                        tick.formatted_as_date(),
-                        tick.formatted_as_time()
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::HIGHLIGHT))
@@ -617,16 +748,16 @@ impl PopupMessage {
                 frame.render_interactive_widget(no_button, buttons_split[1]);
             }
 
-            PopupMessage::PortalFound {
+            Self::PortalFound {
                 player_name,
                 portal_target,
-                tick,
+                timestamp,
             } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Portal: {} {}",
-                        tick.formatted_as_date(),
-                        tick.formatted_as_time()
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::HIGHLIGHT))
@@ -635,7 +766,7 @@ impl PopupMessage {
                 );
 
                 // Select a portal pseudorandomly.
-                let portal = &PORTAL_GIFS[*tick as usize % PORTAL_GIFS.len()];
+                let portal = &PORTAL_GIFS[*timestamp as usize % PORTAL_GIFS.len()];
 
                 if portal.is_empty() {
                     return Err(anyhow!("Invalid portal gif"));
@@ -664,7 +795,7 @@ impl PopupMessage {
                 // Tick::now() returns time as milliseconds. To implement the wanted framerate,
                 // we need to divide by the frame duration in milliseconds
                 let current_frame =
-                    ((Tick::now() - tick) / FRAME_DURATION_MILLIS) as usize % portal.len();
+                    ((Tick::now() - timestamp) / FRAME_DURATION_MILLIS) as usize % portal.len();
 
                 frame.render_widget(
                     Paragraph::new(portal[current_frame].clone()).centered(),
@@ -686,17 +817,17 @@ impl PopupMessage {
                 );
             }
 
-            PopupMessage::ExplorationResult {
+            Self::ExplorationResult {
                 planet_name,
                 resources,
                 players,
-                tick,
+                timestamp,
             } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Exploration result: {} {}",
-                        tick.formatted_as_date(),
-                        tick.formatted_as_time()
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::HIGHLIGHT))
@@ -717,7 +848,7 @@ impl PopupMessage {
                 };
 
                 let m_split = Layout::vertical([
-                    Constraint::Min(3),
+                    Constraint::Fill(1),
                     Constraint::Length(treasure_image_height),
                 ])
                 .split(split[1]);
@@ -763,8 +894,8 @@ impl PopupMessage {
                     // Tick::now() returns time as milliseconds. To implement the wanted framerate,
                     // we need to divide by the frame duration in milliseconds. After the last frame,
                     // we just leave the treasure open rather than looping.
-                    let current_frame = if Tick::now() - tick > TREASURE_GIF_ANIMATION_DELAY {
-                        (((Tick::now() - tick - TREASURE_GIF_ANIMATION_DELAY)
+                    let current_frame = if Tick::now() - timestamp > TREASURE_GIF_ANIMATION_DELAY {
+                        (((Tick::now() - timestamp - TREASURE_GIF_ANIMATION_DELAY)
                             / FRAME_DURATION_MILLIS) as usize)
                             .min(treasure.len() - 1)
                     } else {
@@ -792,18 +923,18 @@ impl PopupMessage {
                 );
             }
 
-            PopupMessage::TeamLanded {
+            Self::TeamLanded {
                 team_name,
                 planet_name,
                 planet_filename,
                 planet_type,
-                tick,
+                timestamp,
             } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Team landed: {} {}",
-                        tick.formatted_as_date(),
-                        tick.formatted_as_time()
+                        timestamp.formatted_as_date(),
+                        timestamp.formatted_as_time()
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::HIGHLIGHT))
@@ -844,7 +975,7 @@ impl PopupMessage {
                 // Tick::now() returns time as milliseconds. To implement the wanted framerate,
                 // we need to divide by the frame duration in milliseconds
                 let current_frame =
-                    ((Tick::now() - tick) / FRAME_DURATION_MILLIS) as usize % planet_gif.len();
+                    ((Tick::now() - timestamp) / FRAME_DURATION_MILLIS) as usize % planet_gif.len();
 
                 frame.render_widget(
                     Paragraph::new(planet_gif_lines[current_frame].clone()).centered(),
@@ -866,12 +997,12 @@ impl PopupMessage {
                 );
             }
 
-            PopupMessage::Tutorial { index, .. } => {
+            Self::Tutorial { index, .. } => {
                 frame.render_widget(
                     Paragraph::new(format!(
                         "Tutorial {}/{}",
                         index + 1,
-                        PopupMessage::MAX_TUTORIAL_PAGE + 1
+                        Self::MAX_TUTORIAL_PAGE + 1
                     ))
                     .bold()
                     .block(default_block().border_style(UiStyle::HIGHLIGHT))
@@ -879,22 +1010,37 @@ impl PopupMessage {
                     split[0],
                 );
 
-                let message = match index {
-                    0 => "Hello pirate! This is your team page.\nHere you can check your pirates, upgrade your spaceship and trade resources on the market.",
-                    1 => "You can navigate around by clicking on the tabs at the top\nor using the arrow keys.",
-                    2 => "To start, you can try to challenge another team to a game,\nor maybe explore around your planet to gather resources.",
-                    3 => "Ultimately, the key is to gather together a great crew and mess around.\n",
-                    _ => "Be sure to check out the Chat in the Swarm panel from time to time.\nHave fun!"
-                };
+                let messages = [
+                     "Hello pirate! This is a brief tutorial to get you started. Check the wiki at wiki.rebels.frittura.org",
+                     "You can navigate around by clicking on the tabs at the top or using the arrow keys.",
+                     "To start, you can try to challenge another team to a game.",
+                     "You can also explore around your planet to gather resources which you can then sell at the market.",
+                     "Once you have enough resources, you can upgrade your spaceship in the Shipyard.",
+                     "You can hire free pirates from the Pirates panel in exchange for satoshi.",
+                     "After you add shooters to your spaceship, you can go in a Space Adventure and try to find Asteroids.",
+                     "Be sure to check out the Chat in the Swarm panel from time to time.\nHave fun!"
+                ];
+
+                let message = messages.get(*index).copied().unwrap_or_default();
+
+                let central_split =
+                    Layout::vertical([Constraint::Fill(1), Constraint::Length(3)]).split(split[1]);
 
                 frame.render_widget(
                     Paragraph::new(message).centered().wrap(Wrap { trim: true }),
-                    split[1].inner(Margin {
-                        horizontal: 1,
-                        vertical: 1,
-                    }),
+                    central_split[0].inner(Margin::new(1, 1)),
                 );
 
+                let close_button = Button::new("Close", UiCallback::CloseUiPopup)
+                    .set_hover_text("Skip the tutorial")
+                    .set_hotkey(ui_key::NO_TO_DIALOG)
+                    .block(default_block().border_style(UiStyle::ERROR))
+                    .set_layer(1);
+
+                let buttons_split =
+                    Layout::horizontal([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)])
+                        .split(split[2]);
+
                 let next_button =
                     Button::new("Next >>", UiCallback::PushTutorialPage { index: index + 1 })
                         .set_hover_text("Next tutorial")
@@ -902,23 +1048,80 @@ impl PopupMessage {
                         .block(default_block().border_style(UiStyle::OK))
                         .set_layer(1);
 
-                let close_button = Button::new("Close", UiCallback::CloseUiPopup)
-                    .set_hover_text("Skip the tutorial")
-                    .set_hotkey(ui_key::NO_TO_DIALOG)
-                    .block(default_block().border_style(UiStyle::ERROR))
-                    .set_layer(1);
-
                 match index {
-                    x if *x < PopupMessage::MAX_TUTORIAL_PAGE => {
-                        let buttons_split =
-                            Layout::horizontal([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)])
-                                .split(split[2]);
+                    2 => {
+                        let games_button =
+                            Button::new("Challenges", UiCallback::TutorialGoToChallenges)
+                                .set_hover_text("Go to available challenges")
+                                .set_hotkey(ui_key::GO_TO_CHALLENGES)
+                                .block(default_block())
+                                .set_layer(1);
+                        frame.render_interactive_widget(games_button, central_split[1]);
+
                         frame.render_interactive_widget(next_button, buttons_split[0]);
                         frame.render_interactive_widget(close_button, buttons_split[1]);
                     }
-                    _ => {
+                    3 => {
+                        let market_button = Button::new("Market", UiCallback::TutorialGoToMarket)
+                            .set_hover_text("Go to market")
+                            .set_hotkey(ui_key::GO_TO_MARKET)
+                            .block(default_block())
+                            .set_layer(1);
+                        frame.render_interactive_widget(market_button, central_split[1]);
+
+                        frame.render_interactive_widget(next_button, buttons_split[0]);
+                        frame.render_interactive_widget(close_button, buttons_split[1]);
+                    }
+                    4 => {
+                        let market_button =
+                            Button::new("Shipyard", UiCallback::TutorialGoToShipyard)
+                                .set_hover_text("Go to shipyard")
+                                .set_hotkey(ui_key::GO_TO_SHIPYARD)
+                                .block(default_block())
+                                .set_layer(1);
+                        frame.render_interactive_widget(market_button, central_split[1]);
+
+                        frame.render_interactive_widget(next_button, buttons_split[0]);
+                        frame.render_interactive_widget(close_button, buttons_split[1]);
+                    }
+                    5 => {
+                        let market_button =
+                            Button::new("Free Pirates", UiCallback::TutorialGoToFreePirates)
+                                .set_hover_text("Go to free pirates")
+                                .set_hotkey(ui_key::GO_TO_FREE_PIRATES)
+                                .block(default_block())
+                                .set_layer(1);
+                        frame.render_interactive_widget(market_button, central_split[1]);
+
+                        frame.render_interactive_widget(next_button, buttons_split[0]);
+                        frame.render_interactive_widget(close_button, buttons_split[1]);
+                    }
+                    6 => {
+                        let market_button =
+                            Button::new("Space Adventure", UiCallback::TutorialGoToSpaceAdventure)
+                                .set_hover_text("Go to space adventure")
+                                .set_hotkey(ui_key::GO_TO_SPACE_ADVENTURE)
+                                .block(default_block())
+                                .set_layer(1);
+                        frame.render_interactive_widget(market_button, central_split[1]);
+
+                        frame.render_interactive_widget(next_button, buttons_split[0]);
+                        frame.render_interactive_widget(close_button, buttons_split[1]);
+                    }
+                    7 => {
+                        let chat_button = Button::new("Chat", UiCallback::TutorialGoToChat)
+                            .set_hover_text("Go to Chat")
+                            .set_hotkey(ui_key::GO_TO_CHAT)
+                            .block(default_block().border_style(UiStyle::NETWORK))
+                            .set_layer(1);
+                        frame.render_interactive_widget(chat_button, central_split[1]);
                         frame.render_interactive_widget(close_button, split[2]);
                     }
+
+                    _ => {
+                        frame.render_interactive_widget(next_button, buttons_split[0]);
+                        frame.render_interactive_widget(close_button, buttons_split[1]);
+                    }
                 }
             }
         }
