@@ -20,9 +20,9 @@ use crate::ui::clickable_list::{ClickableList, ClickableListItem};
 use crate::ui::ui_key;
 use crate::ui::utils::wrap_text;
 use core::fmt::Debug;
-use crossterm::event::{KeyCode, KeyEvent};
 use itertools::Itertools;
 use libp2p::PeerId;
+use ratatui::crossterm::event::{KeyCode, KeyEvent};
 use ratatui::layout::Margin;
 use ratatui::style::Stylize;
 use ratatui::{
@@ -31,7 +31,7 @@ use ratatui::{
     text::{Line, Span},
     widgets::{List, ListItem},
 };
-use std::collections::HashMap;
+use std::collections::{BTreeSet, HashMap};
 use strum_macros::Display;
 use tui_textarea::{CursorMove, TextArea};
 
@@ -48,26 +48,31 @@ pub enum SwarmView {
 }
 
 impl SwarmView {
-    fn next(&self) -> SwarmView {
+    const fn next(&self) -> Self {
         match self {
-            SwarmView::Chat => SwarmView::Requests,
-            SwarmView::Requests => SwarmView::Log,
-            SwarmView::Log => SwarmView::Ranking,
-            SwarmView::Ranking => SwarmView::Chat,
+            Self::Chat => Self::Requests,
+            Self::Requests => Self::Log,
+            Self::Log => Self::Ranking,
+            Self::Ranking => Self::Chat,
         }
     }
 }
 
-#[derive(Debug)]
-pub struct SwarmPanelEvent {
+#[derive(Debug, PartialEq)]
+struct LogEvent {
     timestamp: Tick,
     peer_id: Option<PeerId>,
     text: String,
     level: log::Level,
 }
 
-impl SwarmPanelEvent {
-    pub fn new(timestamp: Tick, peer_id: Option<PeerId>, text: String, level: log::Level) -> Self {
+impl LogEvent {
+    pub const fn new(
+        timestamp: Tick,
+        peer_id: Option<PeerId>,
+        text: String,
+        level: log::Level,
+    ) -> Self {
         Self {
             timestamp,
             peer_id,
@@ -77,6 +82,50 @@ impl SwarmPanelEvent {
     }
 }
 
+#[derive(Debug)]
+struct ChatEvent {
+    timestamp: Tick,
+    peer_id: PeerId,
+    author: String,
+    text: String,
+}
+
+impl ChatEvent {
+    pub const fn new(timestamp: Tick, peer_id: PeerId, author: String, text: String) -> Self {
+        Self {
+            timestamp,
+            peer_id,
+            author,
+            text,
+        }
+    }
+}
+
+impl PartialEq for ChatEvent {
+    fn eq(&self, other: &Self) -> bool {
+        self.timestamp == other.timestamp
+            && self.peer_id == other.peer_id
+            && self.text == other.text
+    }
+}
+
+impl Eq for ChatEvent {}
+
+impl PartialOrd for ChatEvent {
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for ChatEvent {
+    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
+        self.timestamp
+            .cmp(&other.timestamp)
+            .then_with(|| self.peer_id.cmp(&other.peer_id))
+            .then_with(|| self.text.cmp(&other.text))
+    }
+}
+
 #[derive(Debug, Display, Default, PartialEq)]
 enum PanelList {
     #[default]
@@ -87,7 +136,8 @@ enum PanelList {
 #[derive(Debug, Default)]
 pub struct SwarmPanel {
     tick: usize,
-    events: HashMap<SwarmView, Vec<SwarmPanelEvent>>,
+    chat_events: BTreeSet<ChatEvent>,
+    log_events: Vec<LogEvent>,
     view: SwarmView,
     textarea: TextArea<'static>,
     connected_peers: HashMap<PeerId, Tick>,
@@ -102,10 +152,14 @@ pub struct SwarmPanel {
     unread_chat_messages: usize,
     chat_message_index: Option<usize>,
     log_message_index: Option<usize>,
+    chat_message_list: ClickableList<'static>,
+    log_message_list: ClickableList<'static>,
+    should_update_message_list: Option<SwarmView>,
+    emojies_substutions: Vec<(&'static str, &'static str)>,
 }
 
 impl SwarmPanel {
-    pub fn unread_chat_messages(&self) -> usize {
+    pub const fn unread_chat_messages(&self) -> usize {
         self.unread_chat_messages
     }
 
@@ -116,12 +170,33 @@ impl SwarmPanel {
         }
     }
     pub fn new() -> Self {
-        let mut events = HashMap::new();
-        events.insert(SwarmView::Log, vec![]);
-        events.insert(SwarmView::Requests, vec![]);
-        events.insert(SwarmView::Chat, vec![]);
+        let emojies_substutions = vec![
+            (":fire:", "🔥"),
+            (":heart:", "❤️"),
+            (":thumbsup:", "👍"),
+            (":thumbsdown:", "👎"),
+            (":laugh:", "😂"),
+            (":cry:", "😢"),
+            (":star:", "⭐"),
+            (":check:", "✅"),
+            (":x:", "❌"),
+            (":wave:", "👋"),
+            (":clap:", "👏"),
+            (":eyes:", "👀"),
+            (":rocket:", "🚀"),
+            (":trophy:", "🏆"),
+            (":skull:", "💀"),
+            (":100:", "💯"),
+            (":gg:", "🤝"),
+        ];
+
+        let log_message_list = ClickableList::new(vec![]).block(default_block().title("Log"));
+        let chat_message_list = ClickableList::new(vec![]).block(default_block().title("Chat"));
+
         Self {
-            events,
+            emojies_substutions,
+            log_message_list,
+            chat_message_list,
             ..Default::default()
         }
     }
@@ -150,13 +225,16 @@ impl SwarmPanel {
         }
     }
 
-    pub fn push_log_event(&mut self, event: SwarmPanelEvent) {
-        if let Some(last_event) = self
-            .events
-            .get_mut(&SwarmView::Log)
-            .expect("Should have Log events")
-            .last_mut()
-        {
+    pub fn push_log_event(
+        &mut self,
+        timestamp: Tick,
+        peer_id: Option<PeerId>,
+        text: String,
+        level: log::Level,
+    ) {
+        let event = LogEvent::new(timestamp, peer_id, text, level);
+
+        if let Some(last_event) = self.log_events.last_mut() {
             // If we recently pushed the same event, don't push it again,
             // but update the timestamp
             if last_event.peer_id == event.peer_id
@@ -168,26 +246,50 @@ impl SwarmPanel {
             }
         }
 
-        self.events
-            .get_mut(&SwarmView::Log)
-            .expect("Should have Log events")
-            .push(event);
+        self.log_events.push(event);
 
-        self.log_message_index = match self.log_message_index {
-            Some(index) => Some(index + 1),
-            None => Some(0),
-        }
+        self.log_message_index = self
+            .log_message_index
+            .map_or(Some(0), |index| Some(index + 1));
+
+        self.should_update_message_list = Some(SwarmView::Log);
     }
 
-    pub fn push_chat_event(&mut self, event: SwarmPanelEvent) {
-        self.events
-            .get_mut(&SwarmView::Chat)
-            .expect("Should have Chat events")
-            .push(event);
-        self.unread_chat_messages += 1;
-        self.chat_message_index = match self.chat_message_index {
-            Some(index) => Some(index + 1),
-            None => Some(0),
+    pub fn push_chat_event(
+        &mut self,
+        timestamp: Tick,
+        peer_id: PeerId,
+        author: String,
+        message: String,
+    ) {
+        let event = ChatEvent::new(timestamp, peer_id, author, message);
+
+        if self.chat_events.insert(event) {
+            self.unread_chat_messages += 1;
+            self.chat_message_index = self
+                .chat_message_index
+                .map_or(Some(0), |index| Some(index + 1));
+        }
+        self.should_update_message_list = Some(SwarmView::Chat);
+    }
+
+    pub fn push_chat_history(&mut self, entries: Vec<(Tick, PeerId, String, String)>) {
+        let mut any_new = false;
+        for (timestamp, peer_id, author, message) in entries {
+            let event = ChatEvent::new(timestamp, peer_id, author, message);
+            if self.chat_events.insert(event) {
+                any_new = true;
+                self.unread_chat_messages += 1;
+            }
+        }
+
+        if any_new {
+            self.chat_message_index = self
+                .chat_message_index
+                .map_or(Some(self.unread_chat_messages), |index| {
+                    Some(index + self.unread_chat_messages)
+                });
+            self.should_update_message_list = Some(SwarmView::Chat);
         }
     }
 
@@ -195,10 +297,12 @@ impl SwarmPanel {
         self.team_id_to_peer_id.insert(team_id, peer_id);
         self.peer_id_to_team_id.insert(peer_id, team_id);
         self.connected_peers.insert(peer_id, Tick::now());
+        self.should_update_message_list = Some(SwarmView::Chat);
     }
 
     pub fn remove_peer_id(&mut self, peer_id: &PeerId) {
         self.connected_peers.remove(peer_id);
+        self.should_update_message_list = Some(SwarmView::Chat);
     }
 
     fn is_peer_connected(&self, peer_id: &PeerId) -> bool {
@@ -211,7 +315,7 @@ impl SwarmPanel {
         false
     }
 
-    fn build_left_panel(&mut self, frame: &mut UiFrame, world: &World, area: Rect) {
+    fn build_left_panel(&self, frame: &mut UiFrame, world: &World, area: Rect) {
         let split = Layout::vertical([
             Constraint::Length(3),
             Constraint::Length(3),
@@ -278,7 +382,9 @@ impl SwarmPanel {
 
         for (&team_id, peer_id) in self.team_id_to_peer_id.iter() {
             if let Ok(team) = world.teams.get_or_err(&team_id) {
-                let style = if self.is_peer_connected(peer_id) {
+                let style = if team_id == world.own_team_id {
+                    UiStyle::OWN_TEAM
+                } else if self.is_peer_connected(peer_id) {
                     UiStyle::NETWORK
                 } else {
                     UiStyle::DISCONNECTED
@@ -724,8 +830,8 @@ impl SwarmPanel {
                 self.render_team_ranking(frame, world, ranking_split[1]);
             }
 
-            SwarmView::Chat => self.render_event_messages(frame, world, SwarmView::Chat, split[0]),
-            SwarmView::Log => self.render_event_messages(frame, world, SwarmView::Log, split[0]),
+            SwarmView::Chat => self.render_event_messages(frame, SwarmView::Chat, split[0]),
+            SwarmView::Log => self.render_event_messages(frame, SwarmView::Log, split[0]),
         }
 
         // Reset only if current index is equal to max index - 1
@@ -738,107 +844,127 @@ impl SwarmPanel {
         Ok(())
     }
 
-    fn render_event_messages(
-        &self,
-        frame: &mut UiFrame,
-        world: &World,
-        swarm_view: SwarmView,
-        area: Rect,
-    ) {
-        let items = if let Some(events) = self.events.get(&swarm_view) {
-            let mut items = vec![];
-            for event in events.iter() {
-                let timestamp_span = Span::styled(
-                    format!("[{}] ", event.timestamp.formatted_as_time()),
-                    UiStyle::HIGHLIGHT,
-                );
+    fn update_chat_event_list(&mut self, world: &World) {
+        let mut items = vec![];
+        for event in self.chat_events.iter() {
+            let timestamp_span = Span::styled(
+                format!("[{}] ", event.timestamp.formatted_as_time()),
+                UiStyle::HIGHLIGHT,
+            );
 
-                let style = match event.level {
-                    log::Level::Debug => UiStyle::NETWORK,
-                    log::Level::Info => UiStyle::OK,
-                    log::Level::Warn => UiStyle::WARNING,
-                    log::Level::Error => UiStyle::ERROR,
-                    log::Level::Trace => UiStyle::HEADER,
-                };
+            let author = if matches!(self.peer_id_to_team_id.get(&event.peer_id), Some(&id) if id == world.own_team_id)
+            {
+                "You"
+            } else {
+                event.author.as_str()
+            };
 
-                let author_span = match event.peer_id {
-                    Some(peer_id) => {
-                        let from = if let Some(team_id) = self.peer_id_to_team_id.get(&peer_id) {
-                            if let Ok(team) = world.teams.get_or_err(team_id) {
-                                team.name.as_str()
-                            } else {
-                                "Unknown"
-                            }
-                        } else {
-                            "SEED "
-                        };
+            let style = if matches!(self.peer_id_to_team_id.get(&event.peer_id), Some(&id) if id == world.own_team_id)
+            {
+                UiStyle::OWN_TEAM
+            } else if self.is_peer_connected(&event.peer_id) {
+                UiStyle::NETWORK
+            } else {
+                UiStyle::DISCONNECTED
+            };
 
-                        Span::styled(format!("{from}: "), style)
-                    }
-                    None => {
-                        let own_message = if swarm_view == SwarmView::Log {
-                            format!("{:<5}", event.level)
-                        } else {
-                            "You".to_string()
-                        };
+            let author_span = Span::styled(format!("{}: ", author), style);
 
-                        let style = if swarm_view == SwarmView::Log {
-                            style
-                        } else {
-                            UiStyle::OWN_TEAM
-                        };
+            let timestamp_length = timestamp_span.content.len();
+            let incipit_length = timestamp_length + author_span.content.len();
+            // FIXME: we should ideally use area.width and make this adaptive.
+            let message_max_length =
+                (UI_SCREEN_SIZE.0 as usize).saturating_sub(36 + 2 + incipit_length); // 36 is left panel width, extra 2 is because of Block outside.
 
-                        Span::styled(format!("{own_message}: "), style)
-                    }
-                };
+            let text_lines = wrap_text(event.text.as_str(), message_max_length);
+            let mut lines = vec![Line::from(vec![
+                timestamp_span,
+                author_span,
+                Span::raw(text_lines[0].clone()),
+            ])];
 
-                let timestamp_length = timestamp_span.content.len();
-                let incipit_length = timestamp_length + author_span.content.len();
-                let message_max_length = (area.width as usize).saturating_sub(2 + incipit_length); // Extra 2 is because of Block outside.
-
-                let text_lines = wrap_text(event.text.as_str(), message_max_length);
-                let mut lines = vec![Line::from(vec![
-                    timestamp_span,
-                    author_span,
-                    Span::raw(text_lines[0].clone()),
-                ])];
-
-                for text in text_lines.iter().skip(1) {
-                    lines.push(Line::from(format!(
-                        "{}{}",
-                        " ".repeat(incipit_length),
-                        text
-                    )));
-                }
-
-                items.push(ClickableListItem::new(lines));
+            for text in text_lines.iter().skip(1) {
+                lines.push(Line::from(format!(
+                    "{}{}",
+                    " ".repeat(incipit_length),
+                    text
+                )));
             }
 
-            items
-        } else {
-            vec![]
-        };
+            items.push(ClickableListItem::new(lines));
+        }
 
-        let list = ClickableList::new(items);
-        let index = if swarm_view == SwarmView::Chat {
-            self.chat_message_index
-        } else {
-            self.log_message_index
-        };
-        frame.render_stateful_interactive_widget(
-            list.block(default_block().title(swarm_view.to_string())),
-            area,
-            &mut ClickableListState::default().with_selected(index),
-        );
+        self.chat_message_list = ClickableList::new(items).block(default_block().title("Chat"));
     }
 
-    pub fn set_view(&mut self, topic: SwarmView) {
+    fn update_log_event_list(&mut self) {
+        let mut items = vec![];
+        for event in self.log_events.iter() {
+            let timestamp_span = Span::styled(
+                format!("[{}] ", event.timestamp.formatted_as_time()),
+                UiStyle::HIGHLIGHT,
+            );
+
+            let style = match event.level {
+                log::Level::Debug => UiStyle::NETWORK,
+                log::Level::Info => UiStyle::OK,
+                log::Level::Warn => UiStyle::WARNING,
+                log::Level::Error => UiStyle::ERROR,
+                log::Level::Trace => UiStyle::HEADER,
+            };
+
+            let author_span = Span::styled(format!("{:<5}: ", event.level), style);
+
+            let timestamp_length = timestamp_span.content.len();
+            let incipit_length = timestamp_length + author_span.content.len();
+            // FIXME: we should ideally use area.width and make this adaptive.
+            let message_max_length =
+                (UI_SCREEN_SIZE.0 as usize).saturating_sub(36 + 2 + incipit_length); // 36 is left panel width, extra 2 is because of Block outside.
+
+            let text_lines = wrap_text(event.text.as_str(), message_max_length);
+            let mut lines = vec![Line::from(vec![
+                timestamp_span,
+                author_span,
+                Span::raw(text_lines[0].clone()),
+            ])];
+
+            for text in text_lines.iter().skip(1) {
+                lines.push(Line::from(format!(
+                    "{}{}",
+                    " ".repeat(incipit_length),
+                    text
+                )));
+            }
+
+            items.push(ClickableListItem::new(lines));
+        }
+
+        self.log_message_list = ClickableList::new(items).block(default_block().title("Log"));
+    }
+
+    fn render_event_messages(&self, frame: &mut UiFrame, swarm_view: SwarmView, area: Rect) {
+        if swarm_view == SwarmView::Chat {
+            frame.render_stateful_interactive_widget(
+                &self.chat_message_list,
+                area,
+                &mut ClickableListState::default().with_selected(self.chat_message_index),
+            );
+        } else {
+            frame.render_stateful_interactive_widget(
+                &self.log_message_list,
+                area,
+                &mut ClickableListState::default().with_selected(self.log_message_index),
+            )
+        }
+    }
+
+    pub const fn set_view(&mut self, topic: SwarmView) {
         self.view = topic;
     }
 }
 
 impl Screen for SwarmPanel {
-    fn update(&mut self, _world: &World) -> AppResult<()> {
+    fn update(&mut self, world: &World) -> AppResult<()> {
         self.tick += 1;
 
         if self.max_index() == 0 {
@@ -853,6 +979,17 @@ impl Screen for SwarmPanel {
             }
         }
 
+        match self.should_update_message_list {
+            Some(SwarmView::Chat) => {
+                self.update_chat_event_list(world);
+            }
+            Some(SwarmView::Log) => {
+                self.update_log_event_list();
+            }
+            _ => {}
+        }
+        self.should_update_message_list = None;
+
         Ok(())
     }
 
@@ -871,7 +1008,7 @@ impl Screen for SwarmPanel {
         Ok(())
     }
 
-    fn handle_key_events(&mut self, key_event: KeyEvent, _world: &World) -> Option<UiCallback> {
+    fn handle_key_events(&mut self, key_event: KeyEvent, world: &World) -> Option<UiCallback> {
         match key_event.code {
             KeyCode::Up => self.next_index(),
             KeyCode::Down => self.previous_index(),
@@ -893,22 +1030,39 @@ impl Screen for SwarmPanel {
                     .map(|x| x.to_string())
                     .collect();
 
-                let message = lines.iter().join("/n");
+                let mut message = lines.iter().join("/n");
 
                 if message.is_empty() {
+                    // If no message, go to last message
+                    self.chat_message_index = if self.chat_events.is_empty() {
+                        None
+                    } else {
+                        Some(self.chat_events.len() - 1)
+                    };
                     return None;
                 }
 
+                for (from, to) in self.emojies_substutions.iter() {
+                    message = message.replace(from, to);
+                }
+
                 self.textarea.move_cursor(CursorMove::End);
                 self.textarea.delete_line_by_head();
 
-                self.push_chat_event(SwarmPanelEvent::new(
-                    Tick::now(),
-                    None,
+                let own_peer_id = self
+                    .team_id_to_peer_id
+                    .get(&world.own_team_id)
+                    .copied()
+                    .expect("There should be an own peer id.");
+                let own_team = world.get_own_team().expect("There should be an own team.");
+                let timestamp = Tick::now();
+                self.push_chat_event(
+                    timestamp,
+                    own_peer_id,
+                    own_team.name.clone(),
                     message.clone(),
-                    log::Level::Info,
-                ));
-                return Some(UiCallback::SendMessage { message });
+                );
+                return Some(UiCallback::SendMessage { timestamp, message });
             }
             _ => {
                 self.textarea.input(input_from_key_event(key_event));
@@ -940,8 +1094,8 @@ impl SplitPanel for SwarmPanel {
 
     fn max_index(&self) -> usize {
         match self.view {
-            SwarmView::Chat => self.events.get(&SwarmView::Chat).map_or(0, Vec::len),
-            SwarmView::Log => self.events.get(&SwarmView::Log).map_or(0, Vec::len),
+            SwarmView::Chat => self.chat_events.len(),
+            SwarmView::Log => self.log_events.len(),
             SwarmView::Ranking => match self.active_list {
                 PanelList::Players => self.player_ranking.len(),
                 PanelList::Teams => self.team_ranking.len(),
