@@ -30,9 +30,9 @@ use crate::{
     },
 };
 use anyhow::anyhow;
-use crossterm::event::{KeyCode, MouseEvent, MouseEventKind};
 use rand::{seq::IteratorRandom, Rng, SeedableRng};
 use rand_chacha::ChaCha8Rng;
+use ratatui::crossterm::event::{KeyCode, MouseEvent, MouseEventKind};
 use ratatui::layout::Rect;
 use std::collections::HashMap;
 use strum::IntoEnumIterator;
@@ -87,6 +87,12 @@ pub enum UiCallback {
         amount: i32,
         unit_cost: u32,
     },
+    TutorialGoToChat,
+    TutorialGoToChallenges,
+    TutorialGoToMarket,
+    TutorialGoToShipyard,
+    TutorialGoToFreePirates,
+    TutorialGoToSpaceAdventure,
     ChallengeTeam {
         team_id: TeamId,
     },
@@ -231,6 +237,7 @@ pub enum UiCallback {
     Ping,
     Sync,
     SendMessage {
+        timestamp: Tick,
         message: String,
     },
     PushUiPopup {
@@ -343,7 +350,7 @@ impl UiCallback {
                 .players
                 .get_or_err(&player_id)?
                 .team
-                .ok_or(anyhow!("Player {player_id:?} has no team"))?;
+                .ok_or_else(|| anyhow!("Player {player_id:?} has no team"))?;
 
             app.ui.team_panel.reset_view();
 
@@ -656,7 +663,7 @@ impl UiCallback {
             app.ui.set_state(UiState::Main);
             app.ui.push_popup(PopupMessage::Tutorial {
                 index: 0,
-                tick: Tick::now(),
+                timestamp: Tick::now(),
             });
             Ok(None)
         })
@@ -698,7 +705,7 @@ impl UiCallback {
                     tournament.max_participants
                 ),
                 is_skippable: true,
-                tick: Tick::now(),
+                timestamp: Tick::now(),
             });
 
             if let Err(err) = app.network_handler.send_tournament(tournament) {
@@ -733,6 +740,8 @@ impl UiCallback {
             );
 
             app.world.teams.insert(team.id, team);
+            app.world.dirty = true;
+            app.world.dirty_ui = true;
 
             Ok(None)
         })
@@ -922,9 +931,13 @@ impl UiCallback {
         })
     }
 
-    fn send(message: String) -> AppCallback {
+    fn send_message(timestamp: Tick, message: String) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.network_handler.send_message(message.clone())?;
+            let from_peer_id = *app.network_handler.own_peer_id();
+            let own_team = app.world.get_own_team()?;
+            let author = own_team.name.clone();
+            app.network_handler
+                .send_message(timestamp, from_peer_id, author, message.clone())?;
 
             Ok(None)
         })
@@ -933,7 +946,7 @@ impl UiCallback {
     fn name_and_accept_asteroid(name: String, filename: String) -> AppCallback {
         Box::new(move |app: &mut App| {
             let mut own_team = app.world.get_own_team()?.clone();
-            if own_team.asteroid_ids.len() > MAX_NUM_ASTEROID_PER_TEAM {
+            if own_team.asteroid_ids.len() >= MAX_NUM_ASTEROID_PER_TEAM {
                 return Err(anyhow!("Team has reached max number of asteroids."));
             }
 
@@ -1024,7 +1037,7 @@ impl UiCallback {
             app.ui.push_popup(PopupMessage::Ok {
                 message,
                 is_skippable: true,
-                tick: Tick::now(),
+                timestamp: Tick::now(),
             });
 
             app.world.dirty = true;
@@ -1045,7 +1058,7 @@ impl UiCallback {
             } else {
                 app.ui.push_popup(PopupMessage::Error {
                     message: format!("Cannot find asteroid {asteroid_id}"),
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
                 return Ok(None);
             };
@@ -1054,7 +1067,7 @@ impl UiCallback {
             } else {
                 app.ui.push_popup(PopupMessage::Error {
                     message: format!("Cannot find own team {}", app.world.own_team_id),
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
                 return Ok(None);
             };
@@ -1090,7 +1103,7 @@ impl UiCallback {
             } else {
                 app.ui.push_popup(PopupMessage::Error {
                     message: format!("Cannot find asteroid {asteroid_id}"),
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
                 return Ok(None);
             };
@@ -1113,7 +1126,8 @@ impl UiCallback {
                         app.ui.push_log_event(
                             Tick::now(),
                             None,
-                            format!("Cannot finalize space cove upgrade: mismatching planet id."),
+                            "Cannot finalize space cove upgrade: mismatching planet id."
+                                .to_string(),
                             log::Level::Error,
                         );
                         return Ok(None);
@@ -1125,9 +1139,8 @@ impl UiCallback {
                     app.ui.push_log_event(
                         Tick::now(),
                         None,
-                        format!(
-                            "Cannot finalize space cove upgrade: no space cove under construction.",
-                        ),
+                        "Cannot finalize space cove upgrade: no space cove under construction."
+                            .to_string(),
                         log::Level::Error,
                     );
                     return Ok(None);
@@ -1145,7 +1158,7 @@ impl UiCallback {
             app.ui.push_popup(PopupMessage::Ok {
                 message,
                 is_skippable: true,
-                tick: Tick::now(),
+                timestamp: Tick::now(),
             });
             app.world.dirty = true;
             app.world.dirty_ui = true;
@@ -1154,6 +1167,94 @@ impl UiCallback {
         })
     }
 
+    fn return_from_space_adventure() -> AppCallback {
+        Box::new(move |app: &mut App| {
+            app.ui.set_state(UiState::Main);
+            let mut own_team = app.world.get_own_team()?.clone();
+
+            let space = app
+                .world
+                .space_adventure
+                .take()
+                .ok_or_else(|| anyhow!("World should have a space adventure"))?;
+
+            let player = space
+                .get_player()
+                .ok_or_else(|| anyhow!("Space adventure should have a player entity."))?;
+
+            // Update team space adventure data
+
+            own_team.number_of_space_adventures += 1;
+            // At the beginning of the space adventure, we cloned the team resources
+            // so that what is now in possess of the spaceship are the updated team resources,
+            // including potentially resources lost.
+            let mut resources_gathered = vec![];
+            let mut resources_lost = vec![];
+            for resource in Resource::iter() {
+                let old_amount = own_team.resources.value(&resource);
+                let new_amount = player.resources().value(&resource);
+                if old_amount < new_amount {
+                    resources_gathered.push((resource, new_amount - old_amount));
+                } else if old_amount > new_amount && resource != Resource::FUEL {
+                    resources_lost.push((resource, old_amount - new_amount));
+                }
+            }
+
+            let resources_gathered_text = if resources_gathered.is_empty() {
+                "No resources collected!".to_string()
+            } else {
+                format!(
+                    "{} collected.",
+                    resources_gathered
+                        .iter()
+                        .map(|(res, amount)| format!("{amount} {res}"))
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                )
+            };
+
+            let resources_lost_text = if !resources_lost.is_empty() {
+                format!(
+                    "{} lost.",
+                    resources_lost
+                        .iter()
+                        .map(|(res, amount)| format!("{amount} {res}"))
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                )
+            } else {
+                "".to_string()
+            };
+
+            own_team.resources = player.resources().clone();
+            own_team
+                .spaceship
+                .set_current_durability(player.current_durability());
+
+            match own_team.current_location {
+                TeamLocation::OnSpaceAdventure { around } => {
+                    own_team.current_location = TeamLocation::OnPlanet { planet_id: around }
+                }
+                _ => {
+                    return Err(anyhow!("Team should be on a space adventure."));
+                }
+            }
+
+            if let Some(asteroid_type) = space.asteroid_planet_found() {
+                app.ui.push_popup(PopupMessage::AsteroidNameDialog {
+                    timestamp: Tick::now(),
+                    asteroid_type,
+                });
+            }
+
+            app.world.teams.insert(own_team.id, own_team);
+
+            Ok(Some(format!(
+                "Team returned from space adventure:\n{resources_gathered_text}\n{resources_lost_text}"
+            )))
+        })
+    }
+
     pub fn call(&self, app: &mut App) -> AppResult<Option<String>> {
         match self {
             Self::None => Ok(None),
@@ -1161,10 +1262,11 @@ impl UiCallback {
                 app.ui.close_popup();
                 app.ui.push_popup_to_top(PopupMessage::Tutorial {
                     index: *index,
-                    tick: Tick::now(),
+                    timestamp: Tick::now(),
                 });
                 Ok(None)
             }
+
             Self::ToggleUiDebugMode => {
                 app.ui.toggle_data_view();
                 Ok(None)
@@ -1176,6 +1278,48 @@ impl UiCallback {
                 Ok(None)
             }
             Self::GoToTeam { team_id } => Self::go_to_team(*team_id)(app),
+            Self::TutorialGoToChat => {
+                app.ui.swarm_panel.update(&app.world)?;
+                app.ui.swarm_panel.set_view(SwarmView::Chat);
+                app.ui.switch_to(super::ui_screen::UiTab::Swarm);
+
+                Ok(None)
+            }
+            Self::TutorialGoToChallenges => {
+                app.ui.team_panel.update(&app.world)?;
+                app.ui.team_panel.set_view(TeamView::OpenToChallenge);
+                app.ui.switch_to(super::ui_screen::UiTab::Crews);
+
+                Ok(None)
+            }
+            Self::TutorialGoToMarket => {
+                app.ui.my_team_panel.update(&app.world)?;
+                app.ui.my_team_panel.set_view(MyTeamView::Market);
+                app.ui.switch_to(super::ui_screen::UiTab::MyTeam);
+
+                Ok(None)
+            }
+            Self::TutorialGoToShipyard => {
+                app.ui.my_team_panel.update(&app.world)?;
+                app.ui.my_team_panel.set_view(MyTeamView::Shipyard);
+                app.ui.switch_to(super::ui_screen::UiTab::MyTeam);
+
+                Ok(None)
+            }
+            Self::TutorialGoToFreePirates => {
+                app.ui.player_panel.update(&app.world)?;
+                app.ui.player_panel.set_view(PlayerView::FreePirates);
+                app.ui.switch_to(super::ui_screen::UiTab::Pirates);
+
+                Ok(None)
+            }
+            Self::TutorialGoToSpaceAdventure => {
+                app.ui.my_team_panel.update(&app.world)?;
+                app.ui.my_team_panel.set_view(MyTeamView::Info);
+                app.ui.switch_to(super::ui_screen::UiTab::MyTeam);
+
+                Ok(None)
+            }
             Self::GoToPlayer { player_id } => Self::go_to_player(*player_id)(app),
             Self::GoToPlayerTeam { player_id } => Self::go_to_player_team(*player_id)(app),
             Self::GoToLoadedGame { game } => Self::go_to_loaded_game(game.clone())(app),
@@ -1304,13 +1448,22 @@ impl UiCallback {
                 Ok(None)
             }
             Self::QuitGame => {
+                if app.world.space_adventure.is_some() {
+                    let _ = Self::return_from_space_adventure()(app);
+                }
                 app.quit()?;
                 Ok(None)
             }
             #[cfg(feature = "audio")]
             Self::ToggleAudio => {
+                let app_sender = app.get_event_sender();
                 if let Some(player) = app.audio_player.as_mut() {
-                    player.toggle_state()?;
+                    if let Err(err) = player.toggle_state(app_sender) {
+                        app.ui.push_popup_to_top(PopupMessage::Error {
+                            message: format!("Cannot toggle audio: {err}"),
+                            timestamp: Tick::now(),
+                        });
+                    }
                 } else {
                     log::info!("No audio player, cannot toggle it");
                 }
@@ -1319,8 +1472,14 @@ impl UiCallback {
             }
             #[cfg(feature = "audio")]
             Self::PreviousRadio => {
+                let app_sender = app.get_event_sender();
                 if let Some(player) = app.audio_player.as_mut() {
-                    player.previous_radio_stream()?;
+                    if let Err(err) = player.previous_radio_stream(app_sender) {
+                        app.ui.push_popup_to_top(PopupMessage::Error {
+                            message: format!("Cannot toggle audio: {err}"),
+                            timestamp: Tick::now(),
+                        });
+                    }
                 } else {
                     log::info!("No audio player, cannot select previous sample");
                 }
@@ -1328,8 +1487,14 @@ impl UiCallback {
             }
             #[cfg(feature = "audio")]
             Self::NextRadio => {
+                let app_sender = app.get_event_sender();
                 if let Some(player) = app.audio_player.as_mut() {
-                    player.next_radio_stream()?;
+                    if let Err(err) = player.next_radio_stream(app_sender) {
+                        app.ui.push_popup_to_top(PopupMessage::Error {
+                            message: format!("Cannot toggle audio: {err}"),
+                            timestamp: Tick::now(),
+                        });
+                    }
                 } else {
                     log::info!("No audio player, cannot select next sample");
                 }
@@ -1491,7 +1656,7 @@ impl UiCallback {
                         app.ui.push_popup(PopupMessage::PortalFound {
                             player_name: player.info.short_name(),
                             portal_target: portal_target.name.to_string(),
-                            tick: Tick::now(),
+                            timestamp: Tick::now(),
                         });
                     }
                 }
@@ -1550,13 +1715,15 @@ impl UiCallback {
                         own_team.is_organizing_tournament = None;
                     }
 
-                    if tournament.is_team_registered(&app.world.own_team_id) {
+                    if tournament.is_team_registered(&app.world.own_team_id)
+                        || app.world.own_team_id == tournament.organizer_id
+                    {
                         let own_team = app.world.teams.get_mut_or_err(&app.world.own_team_id)?;
                         own_team.tournament_registration_state = TournamentRegistrationState::None;
                         app.ui.push_popup(PopupMessage::Ok {
                             message: format!("{} tournament got cancelled.", tournament.name()),
-                            tick: Tick::now(),
-                            is_skippable: false,
+                            timestamp: Tick::now(),
+                            is_skippable: true,
                         });
                     }
 
@@ -1572,6 +1739,8 @@ impl UiCallback {
                     );
                 }
 
+                app.world.dirty_network = true;
+
                 Ok(None)
             }
 
@@ -1708,7 +1877,9 @@ impl UiCallback {
             } => Self::zoom_to_planet(*planet_id, *zoom_level)(app),
             Self::Ping => Self::ping()(app),
             Self::Sync => Self::sync()(app),
-            Self::SendMessage { message } => Self::send(message.clone())(app),
+            Self::SendMessage { timestamp, message } => {
+                Self::send_message(*timestamp, message.clone())(app)
+            }
             Self::PushUiPopup { popup_message } => {
                 app.ui.push_popup(popup_message.clone());
                 Ok(None)
@@ -1734,7 +1905,8 @@ impl UiCallback {
             Self::StartSpaceAdventure => {
                 app.ui.set_state(UiState::SpaceAdventure);
                 let mut own_team = app.world.get_own_team()?.clone();
-                own_team.can_start_space_adventure()?;
+                let average_tiredness = own_team.average_tiredness(&app.world);
+                own_team.can_start_space_adventure(average_tiredness)?;
 
                 let should_spawn_asteroid = match own_team.current_location {
                     TeamLocation::OnPlanet { planet_id } => {
@@ -1780,96 +1952,22 @@ impl UiCallback {
                         ));
                     }
                 }
+
+                // Apply tiredness cost to crew.
+                for player_id in own_team.player_ids.iter() {
+                    let player = app.world.players.get_mut_or_err(player_id)?;
+                    player.add_tiredness(SPACE_ADVENTURE_TIREDNESS_COST);
+                }
+
+                app.ui.close_popup();
                 app.world.teams.insert(own_team.id, own_team);
                 app.world.space_adventure = Some(space);
+                app.world.dirty_network = true;
+
                 Ok(None)
             }
 
-            Self::ReturnFromSpaceAdventure => {
-                app.ui.set_state(UiState::Main);
-                let mut own_team = app.world.get_own_team()?.clone();
-
-                let space = app
-                    .world
-                    .space_adventure
-                    .take()
-                    .ok_or(anyhow!("World should have a space adventure"))?;
-
-                let player = space
-                    .get_player()
-                    .ok_or(anyhow!("Space adventure should have a player entity."))?;
-
-                // Update team space adventure data
-
-                own_team.number_of_space_adventures += 1;
-                // At the beginning of the space adventure, we cloned the team resources
-                // so that what is now in possess of the spaceship are the updated team resources,
-                // including potentially resources lost.
-                let mut resources_gathered = vec![];
-                let mut resources_lost = vec![];
-                for resource in Resource::iter() {
-                    let old_amount = own_team.resources.value(&resource);
-                    let new_amount = player.resources().value(&resource);
-                    if old_amount < new_amount {
-                        resources_gathered.push((resource, new_amount - old_amount));
-                    } else if old_amount > new_amount && resource != Resource::FUEL {
-                        resources_lost.push((resource, old_amount - new_amount));
-                    }
-                }
-
-                let resources_gathered_text = if resources_gathered.is_empty() {
-                    "No resources collected!".to_string()
-                } else {
-                    format!(
-                        "{} collected.",
-                        resources_gathered
-                            .iter()
-                            .map(|(res, amount)| format!("{amount} {res}"))
-                            .collect::<Vec<_>>()
-                            .join(", ")
-                    )
-                };
-
-                let resources_lost_text = if !resources_lost.is_empty() {
-                    format!(
-                        "{} lost.",
-                        resources_lost
-                            .iter()
-                            .map(|(res, amount)| format!("{amount} {res}"))
-                            .collect::<Vec<_>>()
-                            .join(", ")
-                    )
-                } else {
-                    "".to_string()
-                };
-
-                own_team.resources = player.resources().clone();
-                own_team
-                    .spaceship
-                    .set_current_durability(player.current_durability());
-
-                match own_team.current_location {
-                    TeamLocation::OnSpaceAdventure { around } => {
-                        own_team.current_location = TeamLocation::OnPlanet { planet_id: around }
-                    }
-                    _ => {
-                        return Err(anyhow!("Team should be on a space adventure."));
-                    }
-                }
-
-                if let Some(asteroid_type) = space.asteroid_planet_found() {
-                    app.ui.push_popup(PopupMessage::AsteroidNameDialog {
-                        tick: Tick::now(),
-                        asteroid_type,
-                    });
-                }
-
-                app.world.teams.insert(own_team.id, own_team);
-
-                Ok(Some(format!(
-                    "Team returned from space adventure:\n{resources_gathered_text}\n{resources_lost_text}"
-                )))
-            }
+            Self::ReturnFromSpaceAdventure => Self::return_from_space_adventure()(app),
 
             Self::SpaceAdventurePlayerInput { key_code } => {
                 if let Some(space) = app.world.space_adventure.as_mut() {
@@ -1922,7 +2020,7 @@ pub struct CallbackRegistry {
 }
 
 impl CallbackRegistry {
-    fn contains(rect: &Rect, x: u16, y: u16) -> bool {
+    const fn contains(rect: &Rect, x: u16, y: u16) -> bool {
         rect.x <= x && x < rect.x + rect.width && rect.y <= y && y < rect.y + rect.height
     }
 
@@ -1930,11 +2028,11 @@ impl CallbackRegistry {
         Self::default()
     }
 
-    pub fn set_active_layer(&mut self, layer: usize) {
+    pub const fn set_active_layer(&mut self, layer: usize) {
         self.active_layer = layer;
     }
 
-    pub fn get_active_layer(&self) -> usize {
+    pub const fn get_active_layer(&self) -> usize {
         self.active_layer
     }
 
@@ -1960,15 +2058,15 @@ impl CallbackRegistry {
         self.active_layer = 0;
     }
 
-    pub fn is_hovering(&self, rect: Rect) -> bool {
+    pub const fn is_hovering(&self, rect: Rect) -> bool {
         Self::contains(&rect, self.hovering.0, self.hovering.1)
     }
 
-    pub fn hovering(&self) -> (u16, u16) {
+    pub const fn hovering(&self) -> (u16, u16) {
         self.hovering
     }
 
-    pub fn set_hovering(&mut self, position: (u16, u16)) {
+    pub const fn set_hovering(&mut self, position: (u16, u16)) {
         self.hovering = position;
     }
 
